@@ -67,10 +67,10 @@ nmf = decomposition.NMF(n_components=n_topics).fit(tfidf)
 print "done in %0.3fs." % (time() - t0)
 
 # Inverse the vectorizer vocabulary to be able
-inverse_vocabulary = dict((v, k) for k, v in vectorizer.vocabulary.iteritems())
+feature_names = vectorizer.get_feature_names()
 
 for topic_idx, topic in enumerate(nmf.components_):
     print "Topic #%d:" % topic_idx
-    print " ".join([inverse_vocabulary[i]
+    print " ".join([feature_names[i]
                     for i in topic.argsort()[:-n_top_words - 1:-1]])
     print
