@@ -40,6 +40,11 @@ print "Completeness: %0.3f" % metrics.completeness_score(labels_true, labels)
 print "V-measure: %0.3f" % metrics.v_measure_score(labels_true, labels)
 print "Adjusted Rand Index: %0.3f" % \
     metrics.adjusted_rand_score(labels_true, labels)
+print "Adjusted Mutual Information: %0.3f" % \
+    metrics.adjusted_mutual_info_score(labels_true, labels)
+D = (S / np.min(S))
+print ("Silhouette Coefficient: %0.3f" %
+       metrics.silhouette_score(D, labels, metric='precomputed'))
 
 ##############################################################################
 # Plot result
