@@ -4,7 +4,7 @@ Spectral clustering for image segmentation
 ===========================================
 
 In this example, an image with connected circles is generated and
-spectral clustering is used to separate the circles.
+:ref:`spectral_clustering` is used to separate the circles.
 
 In these settings, the spectral clustering approach solves the problem
 know as 'normalized graph cuts': the image is seen as a graph of
@@ -36,7 +36,7 @@ import pylab as pl
 from sklearn.feature_extraction import image
 from sklearn.cluster import spectral_clustering
 
-################################################################################
+###############################################################################
 l = 100
 x, y = np.indices((l, l))
 
@@ -47,18 +47,18 @@ center4 = (24, 70)
 
 radius1, radius2, radius3, radius4 = 16, 14, 15, 14
 
-circle1 = (x - center1[0])**2 + (y - center1[1])**2 < radius1**2
-circle2 = (x - center2[0])**2 + (y - center2[1])**2 < radius2**2
-circle3 = (x - center3[0])**2 + (y - center3[1])**2 < radius3**2
-circle4 = (x - center4[0])**2 + (y - center4[1])**2 < radius4**2
+circle1 = (x - center1[0]) ** 2 + (y - center1[1]) ** 2 < radius1 ** 2
+circle2 = (x - center2[0]) ** 2 + (y - center2[1]) ** 2 < radius2 ** 2
+circle3 = (x - center3[0]) ** 2 + (y - center3[1]) ** 2 < radius3 ** 2
+circle4 = (x - center4[0]) ** 2 + (y - center4[1]) ** 2 < radius4 ** 2
 
-################################################################################
+###############################################################################
 # 4 circles
 img = circle1 + circle2 + circle3 + circle4
 mask = img.astype(bool)
 img = img.astype(float)
 
-img += 1 + 0.2*np.random.randn(*img.shape)
+img += 1 + 0.2 * np.random.randn(*img.shape)
 
 # Convert the image into a graph with the value of the gradient on the
 # edges.
@@ -66,7 +66,7 @@ graph = image.img_to_graph(img, mask=mask)
 
 # Take a decreasing function of the gradient: we take it weakly
 # dependant from the gradient the segmentation is close to a voronoi
-graph.data = np.exp(-graph.data/graph.data.std())
+graph.data = np.exp(-graph.data / graph.data.std())
 
 # Force the solver to be arpack, since amg is numerically
 # unstable on this example
@@ -77,16 +77,16 @@ label_im[mask] = labels
 pl.matshow(img)
 pl.matshow(label_im)
 
-################################################################################
+###############################################################################
 # 2 circles
 img = circle1 + circle2
 mask = img.astype(bool)
 img = img.astype(float)
 
-img += 1 + 0.2*np.random.randn(*img.shape)
+img += 1 + 0.2 * np.random.randn(*img.shape)
 
 graph = image.img_to_graph(img, mask=mask)
-graph.data = np.exp(-graph.data/graph.data.std())
+graph.data = np.exp(-graph.data / graph.data.std())
 
 labels = spectral_clustering(graph, k=2, mode='arpack')
 label_im = -np.ones(mask.shape)
