@@ -1,7 +1,7 @@
 """
-====================================
-PCA 2D projection of Iris dataset
-====================================
+=======================================================
+Comparison of LDA and PCA 2D projection of Iris dataset
+=======================================================
 
 The Iris dataset represents 3 kind of Iris flowers (Setosa, Versicolour
 and Virginica) with 4 attributes: sepal length, sepal width, petal length
@@ -11,14 +11,18 @@ Principal Component Analysis (PCA) applied to this data identifies the
 combination of attributes (principal components, or directions in the
 feature space) that account for the most variance in the data. Here we
 plot the different samples on the 2 first principal components.
+
+Linear Discriminant Analysis (LDA) tries to identify attributes that
+account for the most variance *between classes*. In particular,
+LDA, in constrast to PCA, is a supervised method, using known class labels.
 """
 print __doc__
 
 import pylab as pl
 
-from scikits.learn import datasets
-from scikits.learn.decomposition import PCA
-from scikits.learn.lda import LDA
+from sklearn import datasets
+from sklearn.decomposition import PCA
+from sklearn.lda import LDA
 
 iris = datasets.load_iris()
 
