@@ -4,26 +4,27 @@ Parameter estimation using grid search with a nested cross-validation
 =====================================================================
 
 The classifier is optimized by "nested" cross-validation using the
-GridSearchCV object.
+:class:`sklearn.grid_search.GridSearchCV` object on a development set
+that comprises only half of the available labeled data.
 
-The performance of the selected parameters is evaluated using
-cross-validation (different than the nested cross-validation that is used
-to select the best classifier).
+The performance of the selected hyper-parameters and trained model is
+then measured on a dedicated evaluation set that was not used during
+the model selection step.
+
+More details on tools available for model selection can be found in the
+sections on :ref:`cross_validation` and :ref:`grid_search`.
 
 """
 print __doc__
 
-from pprint import pprint
-
 from sklearn import datasets
-from sklearn.cross_validation import StratifiedKFold
+from sklearn.cross_validation import train_test_split
 from sklearn.grid_search import GridSearchCV
 from sklearn.metrics import classification_report
 from sklearn.metrics import precision_score
 from sklearn.metrics import recall_score
 from sklearn.svm import SVC
 
-###############################################################################
 # Loading the Digits dataset
 digits = datasets.load_digits()
 
@@ -33,10 +34,10 @@ n_samples = len(digits.images)
 X = digits.images.reshape((n_samples, -1))
 y = digits.target
 
-# split the dataset in two equal part respecting label proportions
-train, test = iter(StratifiedKFold(y, 2)).next()
+# Split the dataset in two equal parts
+X_train, X_test, y_train, y_test = train_test_split(
+    X, y, test_fraction=0.5, random_state=0)
 
-###############################################################################
 # Set the parameters by cross-validation
 tuned_parameters = [{'kernel': ['rbf'], 'gamma': [1e-3, 1e-4],
                      'C': [1, 10, 100, 1000]},
@@ -48,18 +49,31 @@ scores = [
 ]
 
 for score_name, score_func in scores:
-    clf = GridSearchCV(SVC(C=1), tuned_parameters, score_func=score_func)
-    clf.fit(X[train], y[train], cv=StratifiedKFold(y[train], 5))
-    y_true, y_pred = y[test], clf.predict(X[test])
+    print "# Tuning hyper-parameters for %s" % score_name
+    print
 
-    print "Classification report for the best estimator: "
+    clf = GridSearchCV(SVC(C=1), tuned_parameters, score_func=score_func)
+    clf.fit(X_train, y_train, cv=5)
+
+    print "Best parameters set found on development set:"
+    print
     print clf.best_estimator_
-    print "Tuned for '%s' with optimal value: %0.3f" % (
-        score_name, score_func(y_true, y_pred))
+    print
+    print "Grid scores on development set:"
+    print
+    for params, mean_score, scores in clf.grid_scores_:
+        print "%0.3f (+/-%0.03f) for %r" % (
+            mean_score, scores.std() / 2, params)
+    print
+
+    print "Detailed classification report:"
+    print
+    print "The model is trained on the full development set."
+    print "The scores are computed on the full evaluation set."
+    print
+    y_true, y_pred = y_test, clf.predict(X_test)
     print classification_report(y_true, y_pred)
-    print "Grid scores:"
-    pprint(clf.grid_scores_)
     print
 
 # Note the problem is too easy: the hyperparameter plateau is too flat and the
-# output model is the same for precision and recall with ties in quality
+# output model is the same for precision and recall with ties in quality.
