@@ -44,21 +44,19 @@ y_min, y_max = X[:, 1].min() - 1, X[:, 1].max() + 1
 xx, yy = np.meshgrid(np.arange(x_min, x_max, h),
                      np.arange(y_min, y_max, h))
 
-pl.set_cmap(pl.cm.Paired)
-
 # Plot the decision boundary. For that, we will asign a color to each
 # point in the mesh [x_min, m_max]x[y_min, y_max].
 Z = clf.predict(np.c_[xx.ravel(), yy.ravel()])
 # Put the result into a color plot
 Z = Z.reshape(xx.shape)
-pl.set_cmap(pl.cm.Paired)
-cs = pl.contourf(xx, yy, Z)
+cs = pl.contourf(xx, yy, Z, cmap=pl.cm.Paired)
 pl.axis('tight')
 
 # Plot also the training points
-for i, color in zip(clf.classes, colors):
+for i, color in zip(clf.classes_, colors):
     idx = np.where(y == i)
-    pl.scatter(X[idx, 0], X[idx, 1], c=color, label=iris.target_names[i])
+    pl.scatter(X[idx, 0], X[idx, 1], c=color, label=iris.target_names[i],
+            cmap=pl.cm.Paired)
 pl.title("Decision surface of multi-class SGD")
 pl.axis('tight')
 
@@ -76,7 +74,7 @@ def plot_hyperplane(c, color):
     pl.plot([xmin, xmax], [line(xmin), line(xmax)],
             ls="--", color=color)
 
-for i, color in zip(clf.classes, colors):
+for i, color in zip(clf.classes_, colors):
     plot_hyperplane(i, color)
 pl.legend()
 pl.show()
