@@ -12,19 +12,19 @@ print __doc__
 import numpy as np
 import pylab as pl
 from sklearn.linear_model import SGDClassifier
+from sklearn.datasets.samples_generator import make_blobs
 
-# we create 40 separable points
-np.random.seed(0)
-X = np.r_[np.random.randn(20, 2) - [2, 2], np.random.randn(20, 2) + [2, 2]]
-Y = [0] * 20 + [1] * 20
+# we create 50 separable points
+X, Y = make_blobs(n_samples=50, centers=2, random_state=0, cluster_std=0.60)
 
 # fit the model
-clf = SGDClassifier(loss="hinge", alpha=0.01, n_iter=50, fit_intercept=True)
+clf = SGDClassifier(loss="hinge", alpha=0.01, n_iter=200, fit_intercept=True)
 clf.fit(X, Y)
 
 # plot the line, the points, and the nearest vectors to the plane
-xx = np.linspace(-5, 5, 10)
-yy = np.linspace(-5, 5, 10)
+xx = np.linspace(-1, 5, 10)
+yy = np.linspace(-1, 5, 10)
+
 X1, X2 = np.meshgrid(xx, yy)
 Z = np.empty(X1.shape)
 for (i, j), val in np.ndenumerate(X1):
@@ -35,9 +35,8 @@ for (i, j), val in np.ndenumerate(X1):
 levels = [-1.0, 0.0, 1.0]
 linestyles = ['dashed', 'solid', 'dashed']
 colors = 'k'
-pl.set_cmap(pl.cm.Paired)
 pl.contour(X1, X2, Z, levels, colors=colors, linestyles=linestyles)
-pl.scatter(X[:, 0], X[:, 1], c=Y)
+pl.scatter(X[:, 0], X[:, 1], c=Y, cmap=pl.cm.Paired)
 
 pl.axis('tight')
 pl.show()
