@@ -50,13 +50,19 @@ print "Classification score %s (pvalue : %s)" % (score, pvalue)
 
 ###############################################################################
 # View histogram of permutation scores
-pl.hist(permutation_scores, label='Permutation scores')
+pl.hist(permutation_scores, 20, label='Permutation scores')
 ylim = pl.ylim()
-pl.vlines(score, ylim[0], ylim[1], linestyle='--',
-          color='g', linewidth=3, label='Classification Score'
-          ' (pvalue %s)' % pvalue)
-pl.vlines(1.0 / n_classes, ylim[0], ylim[1], linestyle='--',
-          color='k', linewidth=3, label='Luck')
+# BUG: vlines(..., linestyle='--') fails on older versions of matplotlib
+#pl.vlines(score, ylim[0], ylim[1], linestyle='--',
+#          color='g', linewidth=3, label='Classification Score'
+#          ' (pvalue %s)' % pvalue)
+#pl.vlines(1.0 / n_classes, ylim[0], ylim[1], linestyle='--',
+#          color='k', linewidth=3, label='Luck')
+pl.plot(2 * [score], ylim, '--g', linewidth=3,
+        label='Classification Score'
+        ' (pvalue %s)' % pvalue)
+pl.plot(2 * [1. / n_classes], ylim, '--k', linewidth=3, label='Luck')
+
 pl.ylim(ylim)
 pl.legend()
 pl.xlabel('Score')
