@@ -8,7 +8,6 @@ number of features selected with cross-validation.
 """
 print __doc__
 
-import numpy as np
 from sklearn.svm import SVC
 from sklearn.cross_validation import StratifiedKFold
 from sklearn.feature_selection import RFECV
@@ -16,21 +15,14 @@ from sklearn.datasets import make_classification
 from sklearn.metrics import zero_one
 
 # Build a classification task using 3 informative features
-X, y = make_classification(n_samples=1000,
-                           n_features=25,
-                           n_informative=3,
-                           n_redundant=2,
-                           n_repeated=0,
-                           n_classes=8,
-                           n_clusters_per_class=1,
-                           random_state=0)
+X, y = make_classification(n_samples=1000, n_features=25, n_informative=3,
+        n_redundant=2, n_repeated=0, n_classes=8, n_clusters_per_class=1,
+        random_state=0)
 
 # Create the RFE object and compute a cross-validated score.
 svc = SVC(kernel="linear")
-rfecv = RFECV(estimator=svc,
-              step=1,
-              cv=StratifiedKFold(y, 2),
-              loss_func=zero_one)
+rfecv = RFECV(estimator=svc, step=1, cv=StratifiedKFold(y, 2),
+        loss_func=zero_one)
 rfecv.fit(X, y)
 
 print "Optimal number of features : %d" % rfecv.n_features_
