@@ -47,11 +47,10 @@ Z = clf.predict(np.c_[xx.ravel(), yy.ravel()])
 
 # Put the result into a color plot
 Z = Z.reshape(xx.shape)
-pl.set_cmap(pl.cm.Paired)
-pl.pcolormesh(xx, yy, Z)
+pl.pcolormesh(xx, yy, Z, cmap=pl.cm.Paired)
 
 # Plot also the training points
-pl.scatter(X[:, 0], X[:, 1], c=Y)
+pl.scatter(X[:, 0], X[:, 1], c=Y, cmap=pl.cm.Paired)
 pl.title('3-Class classification using Support Vector Machine with custom'
         ' kernel')
 pl.axis('tight')
