@@ -23,10 +23,11 @@ h = .02  # step size in the mesh
 
 # we create an instance of SVM and fit out data. We do not scale our
 # data since we want to plot the support vectors
-svc = svm.SVC(kernel='linear').fit(X, Y)
-rbf_svc = svm.SVC(kernel='rbf', gamma=0.7).fit(X, Y)
-poly_svc = svm.SVC(kernel='poly', degree=3).fit(X, Y)
-lin_svc = svm.LinearSVC().fit(X, Y)
+C = 1.0  # SVM regularization parameter
+svc = svm.SVC(kernel='linear', C=C).fit(X, Y)
+rbf_svc = svm.SVC(kernel='rbf', gamma=0.7, C=C).fit(X, Y)
+poly_svc = svm.SVC(kernel='poly', degree=3, C=C).fit(X, Y)
+lin_svc = svm.LinearSVC(C=C).fit(X, Y)
 
 # create a mesh to plot in
 x_min, x_max = X[:, 0].min() - 1, X[:, 0].max() + 1
@@ -41,8 +42,6 @@ titles = ['SVC with linear kernel',
           'LinearSVC (linear kernel)']
 
 
-pl.set_cmap(pl.cm.Paired)
-
 for i, clf in enumerate((svc, rbf_svc, poly_svc, lin_svc)):
     # Plot the decision boundary. For that, we will asign a color to each
     # point in the mesh [x_min, m_max]x[y_min, y_max].
@@ -51,12 +50,11 @@ for i, clf in enumerate((svc, rbf_svc, poly_svc, lin_svc)):
 
     # Put the result into a color plot
     Z = Z.reshape(xx.shape)
-    pl.set_cmap(pl.cm.Paired)
-    pl.contourf(xx, yy, Z)
+    pl.contourf(xx, yy, Z, cmap=pl.cm.Paired)
     pl.axis('off')
 
     # Plot also the training points
-    pl.scatter(X[:, 0], X[:, 1], c=Y)
+    pl.scatter(X[:, 0], X[:, 1], c=Y, cmap=pl.cm.Paired)
 
     pl.title(titles[i])
 
