@@ -1,3 +1,3 @@
 import warnings
 warnings.warn('scikits.learn namespace is deprecated, please use sklearn instead')
-from sklearn.ball_tree import *
+from sklearn.neighbors.ball_tree import *
