@@ -9,6 +9,7 @@ from scipy import sparse
 import warnings
 
 from .metrics import r2_score
+from .utils import deprecated
 
 
 ###############################################################################
@@ -33,42 +34,66 @@ def clone(estimator, safe=True):
     # XXX: not handling dictionaries
     if estimator_type in (list, tuple, set, frozenset):
         return estimator_type([clone(e, safe=safe) for e in estimator])
-    elif not hasattr(estimator, '_get_params'):
+    elif not hasattr(estimator, 'get_params'):
         if not safe:
             return copy.deepcopy(estimator)
         else:
-            raise ValueError("Cannot clone object '%s' (type %s): "
+            raise TypeError("Cannot clone object '%s' (type %s): "
                     "it does not seem to be a scikit-learn estimator as "
-                    "it does not implement a '_get_params' methods."
+                    "it does not implement a 'get_params' methods."
                     % (repr(estimator), type(estimator)))
     klass = estimator.__class__
-    new_object_params = estimator._get_params(deep=False)
+    new_object_params = estimator.get_params(deep=False)
     for name, param in new_object_params.iteritems():
         new_object_params[name] = clone(param, safe=False)
     new_object = klass(**new_object_params)
-    params_set = new_object._get_params(deep=False)
+    params_set = new_object.get_params(deep=False)
+
+    # quick sanity check of the parameters of the clone
     for name in new_object_params:
         param1 = new_object_params[name]
         param2 = params_set[name]
         if isinstance(param1, np.ndarray):
-            # For ndarrays, we do not test for complete equality
-            equality_test = (param1.shape == param2.shape
-                             and param1.dtype == param2.dtype
-                             and param1[0] == param2[0]
-                             and param1[-1] == param2[-1])
+            # For most ndarrays, we do not test for complete equality
+            if not isinstance(param2, type(param1)):
+                equality_test = False
+            elif (param1.ndim > 0
+                    and param1.shape[0] > 0
+                    and isinstance(param2, np.ndarray)
+                    and param2.ndim > 0
+                    and param2.shape[0] > 0):
+                equality_test = (
+                    param1.shape == param2.shape
+                    and param1.dtype == param2.dtype
+                    # We have to use '.flat' for 2D arrays
+                    and param1.flat[0] == param2.flat[0]
+                    and param1.flat[-1] == param2.flat[-1]
+                )
+            else:
+                equality_test = np.all(param1 == param2)
         elif sparse.issparse(param1):
             # For sparse matrices equality doesn't work
-            equality_test = (param1.__class__ == param2.__class__
-                             and param1.data[0] == param2.data[0]
-                             and param1.data[-1] == param2.data[-1]
-                             and param1.nnz == param2.nnz
-                             and param1.shape == param2.shape)
+            if not sparse.issparse(param2):
+                equality_test = False
+            elif param1.size == 0 or param2.size == 0:
+                equality_test = (
+                    param1.__class__ == param2.__class__
+                    and param1.size == 0
+                    and param2.size == 0
+                )
+            else:
+                equality_test = (
+                    param1.__class__ == param2.__class__
+                    and param1.data[0] == param2.data[0]
+                    and param1.data[-1] == param2.data[-1]
+                    and param1.nnz == param2.nnz
+                    and param1.shape == param2.shape
+                )
         else:
             equality_test = new_object_params[name] == params_set[name]
-        assert equality_test, (
-                'Cannot clone object %s, as the constructor does not '
-                'seem to set parameter %s' % (estimator, name)
-            )
+        if not equality_test:
+            raise RuntimeError('Cannot clone object %s, as the constructor '
+                'does not seem to set parameter %s' % (estimator, name))
 
     return new_object
 
@@ -147,10 +172,10 @@ class BaseEstimator(object):
             # introspect the constructor arguments to find the model parameters
             # to represent
             args, varargs, kw, default = inspect.getargspec(init)
-            assert varargs is None, (
-                'scikit learn estimators should always specify their '
-                'parameters in the signature of their init (no varargs).'
-            )
+            if not varargs is None:
+                raise RuntimeError('scikit learn estimators should always '
+                        'specify their parameters in the signature of '
+                        'their init (no varargs).')
             # Remove 'self'
             # XXX: This is going to fail if the init is a staticmethod, but
             # who would do this?
@@ -161,7 +186,11 @@ class BaseEstimator(object):
         args.sort()
         return args
 
+    @deprecated("to be removed in v0.12; use get_params() instead")
     def _get_params(self, deep=True):
+        return self.get_params(deep)
+
+    def get_params(self, deep=True):
         """Get parameters for the estimator
 
         Parameters
@@ -173,8 +202,8 @@ class BaseEstimator(object):
         out = dict()
         for key in self._get_param_names():
             value = getattr(self, key, None)
-            if deep and hasattr(value, '_get_params'):
-                deep_items = value._get_params().items()
+            if deep and hasattr(value, 'get_params'):
+                deep_items = value.get_params().items()
                 out.update((key + '__' + k, val) for k, val in deep_items)
             out[key] = value
         return out
@@ -194,17 +223,18 @@ class BaseEstimator(object):
         if not params:
             # Simple optimisation to gain speed (inspect is slow)
             return
-        valid_params = self._get_params(deep=True)
+        valid_params = self.get_params(deep=True)
         for key, value in params.iteritems():
             split = key.split('__', 1)
             if len(split) > 1:
                 # nested objects case
                 name, sub_name = split
-                assert name in valid_params, ('Invalid parameter %s '
-                                              'for estimator %s' %
-                                             (name, self))
+                if not name in valid_params:
+                    raise ValueError('Invalid parameter %s for estimator %s'
+                            % (name, self))
                 sub_object = valid_params[name]
-                assert hasattr(sub_object, '_get_params'), (
+                if not hasattr(sub_object, 'get_params'):
+                    raise TypeError(
                     'Parameter %s of %s is not an estimator, cannot set '
                     'sub parameter %s' %
                         (sub_name, self.__class__.__name__, sub_name)
@@ -212,9 +242,9 @@ class BaseEstimator(object):
                 sub_object.set_params(**{sub_name: value})
             else:
                 # simple objects case
-                assert key in valid_params, ('Invalid parameter %s '
-                                              'for estimator %s' %
-                                             (key, self.__class__.__name__))
+                if not key in valid_params:
+                    raise ValueError('Invalid parameter %s ' 'for estimator %s'
+                            % (key, self.__class__.__name__))
                 setattr(self, key, value)
         return self
 
@@ -229,7 +259,7 @@ class BaseEstimator(object):
         class_name = self.__class__.__name__
         return '%s(%s)' % (
                 class_name,
-                _pprint(self._get_params(deep=False),
+                _pprint(self.get_params(deep=False),
                         offset=len(class_name),
                 ),
             )
@@ -238,7 +268,7 @@ class BaseEstimator(object):
         class_name = self.__class__.__name__
         return '%s(%s)' % (
                 class_name,
-                _pprint(self._get_params(deep=True),
+                _pprint(self.get_params(deep=True),
                         offset=len(class_name),
                         printer=str,
                 ),
