@@ -22,13 +22,23 @@ def affinity_propagation(S, p=None, convit=30, max_iter=200, damping=0.5,
 
     S: array [n_points, n_points]
         Matrix of similarities between points
+
     p: array [n_points,] or float, optional
-        Preferences for each point
+        Preferences for each point - points with larger values of
+        preferences are more likely to be chosen as exemplars. The number of
+        exemplars, ie of clusters, is influenced by the input preferences
+        value. If the preferences are not passed as arguments, they will be
+        set to the median of the input similarities (resulting in a moderate
+        number of clusters). For a smaller amount of clusters, this can be set
+        to the minimum value of the similarities.
+
     damping : float, optional
         Damping factor
+
     copy: boolean, optional
         If copy is False, the affinity matrix is modified inplace by the
         algorithm, for memory efficiency
+
     verbose: boolean, optional
         The verbosity level
 
@@ -45,16 +55,17 @@ def affinity_propagation(S, p=None, convit=30, max_iter=200, damping=0.5,
     -----
     See examples/plot_affinity_propagation.py for an example.
 
-    **References**:
+    References
+    ----------
     Brendan J. Frey and Delbert Dueck, "Clustering by Passing Messages
     Between Data Points", Science Feb. 2007
-
     """
     S = as_float_array(S, copy=copy)
 
     n_points = S.shape[0]
 
-    assert S.shape[0] == S.shape[1]
+    if S.shape[0] != S.shape[1]:
+        raise ValueError("S must be a square array (shape=%r)" % S.shape)
 
     if p is None:
         p = np.median(S)
@@ -159,7 +170,6 @@ class AffinityPropagation(BaseEstimator):
 
     Parameters
     ----------
-
     damping : float, optional
         Damping factor
 
@@ -176,7 +186,6 @@ class AffinityPropagation(BaseEstimator):
 
     Attributes
     ----------
-
     `cluster_centers_indices_` : array, [n_clusters]
         Indices of cluster centers
 
@@ -187,13 +196,14 @@ class AffinityPropagation(BaseEstimator):
     -----
     See examples/plot_affinity_propagation.py for an example.
 
-    **References**:
+    The algorithmic complexity of affinity propagation is quadratic
+    in the number of points.
+
+    References
+    ----------
 
     Brendan J. Frey and Delbert Dueck, "Clustering by Passing Messages
     Between Data Points", Science Feb. 2007
-
-    The algorithmic complexity of affinity propagation is quadratic
-    in the number of points.
     """
 
     def __init__(self, damping=.5, max_iter=200, convit=30, copy=True):
@@ -203,17 +213,24 @@ class AffinityPropagation(BaseEstimator):
         self.copy = copy
 
     def fit(self, S, p=None):
-        """Compute MeanShift clustering.
+        """Compute affinity propagation clustering.
 
         Parameters
         ----------
 
         S: array [n_points, n_points]
             Matrix of similarities between points
+
         p: array [n_points,] or float, optional
-            Preferences for each point
+            Preferences for each point - points with larger values of
+            preferences are more likely to be chosen as exemplars. The number
+            of exemplars, ie of clusters, is influenced by the input
+            preferences value. If the preferences are not passed as arguments,
+            they will be set to the median of the input similarities.
+
         damping : float, optional
             Damping factor
+
         copy: boolean, optional
             If copy is False, the affinity matrix is modified inplace by the
             algorithm, for memory efficiency
