@@ -6,6 +6,7 @@ Author : Vincent Michel, 2010
 
 import numpy as np
 from scipy.cluster import hierarchy
+from nose.tools import assert_true
 
 from sklearn.cluster import Ward, WardAgglomeration, ward_tree
 from sklearn.cluster.hierarchical import _hc_cut
@@ -16,68 +17,68 @@ def test_structured_ward_tree():
     """
     Check that we obtain the correct solution for structured ward tree.
     """
-    np.random.seed(0)
+    rnd = np.random.RandomState(0)
     mask = np.ones([10, 10], dtype=np.bool)
-    X = np.random.randn(50, 100)
+    X = rnd.randn(50, 100)
     connectivity = grid_to_graph(*mask.shape)
     children, n_components, n_leaves = ward_tree(X.T, connectivity)
     n_nodes = 2 * X.shape[1] - 1
-    assert(len(children) + n_leaves == n_nodes)
+    assert_true(len(children) + n_leaves == n_nodes)
 
 
 def test_unstructured_ward_tree():
     """
     Check that we obtain the correct solution for unstructured ward tree.
     """
-    np.random.seed(0)
-    X = np.random.randn(50, 100)
+    rnd = np.random.RandomState(0)
+    X = rnd.randn(50, 100)
     children, n_nodes, n_leaves = ward_tree(X.T)
     n_nodes = 2 * X.shape[1] - 1
-    assert(len(children) + n_leaves == n_nodes)
+    assert_true(len(children) + n_leaves == n_nodes)
 
 
 def test_height_ward_tree():
     """
     Check that the height of ward tree is sorted.
     """
-    np.random.seed(0)
+    rnd = np.random.RandomState(0)
     mask = np.ones([10, 10], dtype=np.bool)
-    X = np.random.randn(50, 100)
+    X = rnd.randn(50, 100)
     connectivity = grid_to_graph(*mask.shape)
     children, n_nodes, n_leaves = ward_tree(X.T, connectivity)
     n_nodes = 2 * X.shape[1] - 1
-    assert(len(children) + n_leaves == n_nodes)
+    assert_true(len(children) + n_leaves == n_nodes)
 
 
 def test_ward_clustering():
     """
     Check that we obtain the correct number of clusters with Ward clustering.
     """
-    np.random.seed(0)
+    rnd = np.random.RandomState(0)
     mask = np.ones([10, 10], dtype=np.bool)
-    X = np.random.randn(100, 50)
+    X = rnd.randn(100, 50)
     connectivity = grid_to_graph(*mask.shape)
     clustering = Ward(n_clusters=10, connectivity=connectivity)
     clustering.fit(X)
-    assert(np.size(np.unique(clustering.labels_)) == 10)
+    assert_true(np.size(np.unique(clustering.labels_)) == 10)
 
 
 def test_ward_agglomeration():
     """
     Check that we obtain the correct solution in a simplistic case
     """
-    np.random.seed(0)
+    rnd = np.random.RandomState(0)
     mask = np.ones([10, 10], dtype=np.bool)
-    X = np.random.randn(50, 100)
+    X = rnd.randn(50, 100)
     connectivity = grid_to_graph(*mask.shape)
     ward = WardAgglomeration(n_clusters=5, connectivity=connectivity)
     ward.fit(X)
-    assert(np.size(np.unique(ward.labels_)) == 5)
+    assert_true(np.size(np.unique(ward.labels_)) == 5)
 
     Xred = ward.transform(X)
-    assert(Xred.shape[1] == 5)
+    assert_true(Xred.shape[1] == 5)
     Xfull = ward.inverse_transform(Xred)
-    assert(np.unique(Xfull[0]).size == 5)
+    assert_true(np.unique(Xfull[0]).size == 5)
 
 
 def assess_same_labelling(cut1, cut2):
@@ -89,7 +90,7 @@ def assess_same_labelling(cut1, cut2):
         ecut = np.zeros((n, k))
         ecut[np.arange(n), cut] = 1
         co_clust.append(np.dot(ecut, ecut.T))
-    assert((co_clust[0] == co_clust[1]).all())
+    assert_true((co_clust[0] == co_clust[1]).all())
 
 
 def test_scikit_vs_scipy():
@@ -97,10 +98,11 @@ def test_scikit_vs_scipy():
     """
     from scipy.sparse import lil_matrix
     n, p, k = 10, 5, 3
+    rnd = np.random.RandomState(0)
 
     connectivity = lil_matrix(np.ones((n, n)))
     for i in range(5):
-        X = .1 * np.random.normal(size=(n, p))
+        X = .1 * rnd.normal(size=(n, p))
         X -= 4 * np.arange(n)[:, np.newaxis]
         X -= X.mean(axis=1)[:, np.newaxis]
 
@@ -119,7 +121,7 @@ def test_connectivity_popagation():
     Check that connectivity in the ward tree is propagated correctly during
     merging.
     """
-    from sklearn.neighbors import kneighbors_graph
+    from sklearn.neighbors import NearestNeighbors
 
     X = np.array([(.014, .120), (.014, .099), (.014, .097),
                   (.017, .153), (.017, .153), (.018, .153),
@@ -127,8 +129,8 @@ def test_connectivity_popagation():
                   (.018, .153), (.018, .153), (.018, .153),
                   (.018, .152), (.018, .149), (.018, .144),
                  ])
-
-    connectivity = kneighbors_graph(X, n_neighbors=10)
+    nn = NearestNeighbors(n_neighbors=10, warn_on_equidistant=False).fit(X)
+    connectivity = nn.kneighbors_graph(X)
     ward = Ward(n_clusters=4, connectivity=connectivity)
     # If changes are not propagated correctly, fit crashes with an
     # IndexError
