@@ -1,14 +1,18 @@
 """Testing for K-means"""
 
 import numpy as np
+import warnings
 from scipy import sparse as sp
 from numpy.testing import assert_equal
 from numpy.testing import assert_array_equal
 from numpy.testing import assert_array_almost_equal
+from nose import SkipTest
 from nose.tools import assert_almost_equal
 from nose.tools import assert_raises
 from nose.tools import assert_true
 
+from sklearn.utils.testing import assert_greater
+from sklearn.utils.testing import assert_less
 from sklearn.metrics.cluster import v_measure_score
 from sklearn.cluster import KMeans
 from sklearn.cluster import MiniBatchKMeans
@@ -38,6 +42,16 @@ def test_square_norms():
                               x_squared_norms_from_csr, 5)
 
 
+def test_kmeans_dtype():
+    rnd = np.random.RandomState(0)
+    X = rnd.normal(size=(40, 2))
+    X = (X * 10).astype(np.uint8)
+    km = KMeans(n_init=1).fit(X)
+    with warnings.catch_warnings(record=True) as w:
+        assert_array_equal(km.labels_, km.predict(X))
+        assert_equal(len(w), 1)
+
+
 def test_labels_assignement_and_inertia():
     # pure numpy implementation as easily auditable reference gold
     # implementation
@@ -96,13 +110,13 @@ def test_minibatch_update_consistency():
     old_inertia, incremental_diff = _mini_batch_step(
         X_mb, x_mb_squared_norms, new_centers, counts,
         buffer, 1)
-    assert_true(old_inertia > 0.0)
+    assert_greater(old_inertia, 0.0)
 
     # compute the new inertia on the same batch to check that it decreased
     labels, new_inertia = _labels_inertia(
         X_mb, x_mb_squared_norms, new_centers)
-    assert_true(new_inertia > 0.0)
-    assert_true(new_inertia < old_inertia)
+    assert_greater(new_inertia, 0.0)
+    assert_less(new_inertia, old_inertia)
 
     # check that the incremental difference computation is matching the
     # final observed value
@@ -113,13 +127,13 @@ def test_minibatch_update_consistency():
     old_inertia_csr, incremental_diff_csr = _mini_batch_step(
         X_mb_csr, x_mb_squared_norms_csr, new_centers_csr, counts_csr,
         buffer_csr, 1)
-    assert_true(old_inertia_csr > 0.0)
+    assert_greater(old_inertia_csr, 0.0)
 
     # compute the new inertia on the same batch to check that it decreased
     labels_csr, new_inertia_csr = _labels_inertia(
         X_mb_csr, x_mb_squared_norms_csr, new_centers_csr)
-    assert_true(new_inertia_csr > 0.0)
-    assert_true(new_inertia_csr < old_inertia_csr)
+    assert_greater(new_inertia_csr, 0.0)
+    assert_less(new_inertia_csr, old_inertia_csr)
 
     # check that the incremental difference computation is matching the
     # final observed value
@@ -135,6 +149,8 @@ def test_minibatch_update_consistency():
 
 
 def _check_fitted_model(km):
+    # check that the number of clusters centers and distinct labels match
+    # the expectation
     centers = km.cluster_centers_
     assert_equal(centers.shape, (n_clusters, n_features))
 
@@ -143,7 +159,7 @@ def _check_fitted_model(km):
 
     # check that the labels assignements are perfect (up to a permutation)
     assert_equal(v_measure_score(true_labels, labels), 1.0)
-    assert_true(km.inertia_ > 0.0)
+    assert_greater(km.inertia_, 0.0)
 
     # check error on dataset being too small
     assert_raises(ValueError, km.fit, [[0., 1.]])
@@ -154,6 +170,22 @@ def test_k_means_plus_plus_init():
     _check_fitted_model(k_means)
 
 
+def _get_mac_os_version():
+    import platform
+    mac_version, _, _ = platform.mac_ver()
+    if mac_version:
+        # turn something like '10.7.3' into '10.7'
+        return '.'.join(mac_version.split('.')[:2])
+
+
+def test_k_means_plus_plus_init_2_jobs():
+    if _get_mac_os_version() == '10.7':
+        raise SkipTest('Multi-process bug in Mac OS X Lion (see issue #636)')
+    k_means = KMeans(init="k-means++", k=n_clusters, n_jobs=2,
+                     random_state=42).fit(X)
+    _check_fitted_model(k_means)
+
+
 def test_k_means_plus_plus_init_sparse():
     k_means = KMeans(init="k-means++", k=n_clusters, random_state=42)
     k_means.fit(X_csr)
@@ -203,6 +235,16 @@ def test_mb_k_means_plus_plus_init_sparse_matrix():
     _check_fitted_model(mb_k_means)
 
 
+def test_minibatch_init_with_large_k():
+    mb_k_means = MiniBatchKMeans(init='k-means++', init_size=10, k=20)
+    # Check that a warning is raised, as the number clusters is larger
+    # than the init_size
+    with warnings.catch_warnings(record=True) as warn_queue:
+        mb_k_means.fit(X)
+
+    assert_equal(len(warn_queue), 1)
+
+
 def test_minibatch_k_means_random_init_dense_array():
     # increase n_init to make random init stable enough
     mb_k_means = MiniBatchKMeans(init="random", k=n_clusters,
@@ -250,11 +292,31 @@ def test_mini_batch_k_means_random_init_partial_fit():
     assert_equal(v_measure_score(true_labels, labels), 1.0)
 
 
+def test_minibatch_default_init_size():
+    mb_k_means = MiniBatchKMeans(init=centers.copy(), k=n_clusters,
+                                 batch_size=10, random_state=42).fit(X)
+    assert_equal(mb_k_means.init_size_, 3 * mb_k_means.batch_size)
+    _check_fitted_model(mb_k_means)
+
+
+def test_minibatch_set_init_size():
+    mb_k_means = MiniBatchKMeans(init=centers.copy(), k=n_clusters,
+                                 init_size=666, random_state=42).fit(X)
+    assert_equal(mb_k_means.init_size, 666)
+    assert_equal(mb_k_means.init_size_, n_samples)
+    _check_fitted_model(mb_k_means)
+
+
 def test_k_means_invalid_init():
     k_means = KMeans(init="invalid", n_init=1, k=n_clusters)
     assert_raises(ValueError, k_means.fit, X)
 
 
+def test_mini_match_k_means_invalid_init():
+    k_means = MiniBatchKMeans(init="invalid", n_init=1, k=n_clusters)
+    assert_raises(ValueError, k_means.fit, X)
+
+
 def test_k_means_copyx():
     """Check if copy_x=False returns nearly equal X after de-centering."""
     my_X = X.copy()
@@ -288,7 +350,9 @@ def test_k_means_non_collapsed():
 
 
 def test_predict():
-    k_means = KMeans(k=n_clusters, random_state=42).fit(X)
+    k_means = KMeans(k=n_clusters, random_state=42)
+
+    k_means.fit(X)
 
     # sanity check: predict centroid labels
     pred = k_means.predict(k_means.cluster_centers_)
@@ -296,13 +360,17 @@ def test_predict():
 
     # sanity check: re-predict labeling for training set samples
     pred = k_means.predict(X)
-    assert_array_equal(k_means.predict(X), k_means.labels_)
+    assert_array_equal(pred, k_means.labels_)
+
+    # re-predict labels for training set using fit_predict
+    pred = k_means.fit_predict(X)
+    assert_array_equal(pred, k_means.labels_)
 
 
 def test_score():
     s1 = KMeans(k=n_clusters, max_iter=1, random_state=42).fit(X).score(X)
     s2 = KMeans(k=n_clusters, max_iter=10, random_state=42).fit(X).score(X)
-    assert_true(s2 > s1)
+    assert_greater(s2, s1)
 
 
 def test_predict_minibatch_dense_input():
@@ -384,7 +452,7 @@ def test_transform():
         assert_equal(X_new[c, c], 0)
         for c2 in range(n_clusters):
             if c != c2:
-                assert_true(X_new[c, c2] > 0)
+                assert_greater(X_new[c, c2], 0)
 
 
 def test_n_init():
