@@ -36,22 +36,26 @@ def test_mean_shift():
     n_clusters_ = len(labels_unique)
     assert_equal(n_clusters_, n_clusters)
 
+
 def test_bin_seeds():
     """
-    Test the bin seeding technique which can be used in the mean shift algorithm
+    Test the bin seeding technique which can be used in the mean shift
+    algorithm
     """
     # Data is just 6 points in the plane
     X = np.array([[1., 1.], [1.5, 1.5], [1.8, 1.2],
                   [2., 1.], [2.1, 1.1], [0., 0.]])
 
-    # With a bin coarseness of 1.0 and min_bin_freq of 1, 3 bins should be found
-    ground_truth = set([(1.,1.), (2.,1.), (0.,0.)])
+    # With a bin coarseness of 1.0 and min_bin_freq of 1, 3 bins should be
+    # found
+    ground_truth = set([(1., 1.), (2., 1.), (0., 0.)])
     test_bins = get_bin_seeds(X, 1, 1)
     test_result = set([tuple(p) for p in test_bins])
     assert_true(len(ground_truth.symmetric_difference(test_result)) == 0)
 
-    # With a bin coarseness of 1.0 and min_bin_freq of 2, 2 bins should be found
-    ground_truth = set([(1.,1.), (2.,1.)])
+    # With a bin coarseness of 1.0 and min_bin_freq of 2, 2 bins should be
+    # found
+    ground_truth = set([(1., 1.), (2., 1.)])
     test_bins = get_bin_seeds(X, 1, 2)
     test_result = set([tuple(p) for p in test_bins])
     assert_true(len(ground_truth.symmetric_difference(test_result)) == 0)
@@ -60,5 +64,3 @@ def test_bin_seeds():
     test_bins = get_bin_seeds(X, 0.01, 1)
     test_result = set([tuple(p) for p in test_bins])
     assert_true(len(test_result) == 6)
-
-
