@@ -16,11 +16,12 @@ libsvm command line programs.
 # License: Simple BSD.
 
 import numpy as np
+import scipy.sparse as sp
 from ._svmlight_format import _load_svmlight_file
 
 
 def load_svmlight_file(f, n_features=None, dtype=np.float64,
-                       multilabel=False):
+                       multilabel=False, zero_based="auto"):
     """Load datasets in the svmlight / libsvm format into sparse CSR matrix
 
     This format is a text-based format, with one sample per line. It does
@@ -60,6 +61,13 @@ def load_svmlight_file(f, n_features=None, dtype=np.float64,
         Samples may have several labels each (see
         http://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multilabel.html)
 
+    zero_based: boolean or "auto", optional
+        Whether column indices in f are zero-based (True) or one-based
+        (False). If set to "auto", a heuristic check is applied to determine
+        this from the file contents. Both kinds of files occur "in the wild",
+        but they are unfortunately not self-identifying. Using "auto" or True
+        should always be safe.
+
     Returns
     -------
     (X, y)
@@ -67,15 +75,25 @@ def load_svmlight_file(f, n_features=None, dtype=np.float64,
     where X is a scipy.sparse matrix of shape (n_samples, n_features),
           y is a ndarray of shape (n_samples,), or, in the multilabel case,
           a list of tuples of length n_samples.
+
+    See also
+    --------
+    load_svmlight_files: similar function for loading multiple files in this
+    format, enforcing the same number of features/columns on all of them.
     """
+    return tuple(load_svmlight_files([f], n_features, dtype, multilabel,
+                                     zero_based))
+
+
+def _open_and_load(f, dtype, multilabel, zero_based):
     if hasattr(f, "read"):
-        return _load_svmlight_file(f, n_features, dtype, multilabel)
-    with open(f, 'rb') as f:
-        return _load_svmlight_file(f, n_features, dtype, multilabel)
+        return _load_svmlight_file(f, dtype, multilabel, zero_based)
+    with open(f, "rb") as f:
+        return _load_svmlight_file(f, dtype, multilabel, zero_based)
 
 
 def load_svmlight_files(files, n_features=None, dtype=np.float64,
-                        multilabel=False):
+                        multilabel=False, zero_based="auto"):
     """Load dataset from multiple files in SVMlight format
 
     This function is equivalent to mapping load_svmlight_file over a list of
@@ -90,15 +108,19 @@ def load_svmlight_files(files, n_features=None, dtype=np.float64,
 
     n_features: int or None
         The number of features to use. If None, it will be inferred from the
-        first file. This argument is useful to load several files that are
-        subsets of a bigger sliced dataset: each subset might not have
-        examples of every feature, hence the inferred shape might vary from
-        one slice to another.
+        maximum column index occurring in any of the files.
 
     multilabel: boolean, optional
         Samples may have several labels each (see
         http://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/multilabel.html)
 
+    zero_based: boolean or "auto", optional
+        Whether column indices in files are zero-based (True) or one-based
+        (False). If set to "auto", a heuristic check is applied to determine
+        this from the files' contents. Both kinds of files occur "in the wild",
+        but they are unfortunately not self-identifying. Using "auto" or True
+        should always be safe.
+
     Returns
     -------
     [X1, y1, ..., Xn, yn]
@@ -110,36 +132,46 @@ def load_svmlight_files(files, n_features=None, dtype=np.float64,
     When fitting a model to a matrix X_train and evaluating it against a
     matrix X_test, it is essential that X_train and X_test have the same
     number of features (X_train.shape[1] == X_test.shape[1]). This may not
-    be the case if you load them with load_svmlight_file separately.
+    be the case if you load the files individually with load_svmlight_file.
 
     See also
     --------
     load_svmlight_file
     """
-    files = iter(files)
-    result = list(load_svmlight_file(files.next(), n_features, dtype))
-    n_features = result[0].shape[1]
+    r = [_open_and_load(f, dtype, multilabel, bool(zero_based)) for f in files]
 
-    for f in files:
-        result += load_svmlight_file(f, n_features, dtype, multilabel)
+    if zero_based is False \
+     or zero_based == "auto" and all(np.min(indices) > 0
+                                     for _, indices, _, _ in r):
+        for _, indices, _, _ in r:
+            indices -= 1
+
+    if n_features is None:
+        n_features = max(indices.max() for _, indices, _, _ in r) + 1
+
+    result = []
+    for data, indices, indptr, y in r:
+        shape = (indptr.shape[0] - 1, n_features)
+        result += sp.csr_matrix((data, indices, indptr), shape), y
 
     return result
 
 
-def _dump_svmlight(X, y, f):
+def _dump_svmlight(X, y, f, zero_based):
     if X.shape[0] != y.shape[0]:
         raise ValueError("X.shape[0] and y.shape[0] should be the same, "
                          "got: %r and %r instead." % (X.shape[0], y.shape[0]))
 
     is_sp = int(hasattr(X, "tocsr"))
 
+    one_based = not zero_based
     for i in xrange(X.shape[0]):
-        s = u" ".join([u"%d:%f" % (j + 1, X[i, j])
+        s = u" ".join([u"%d:%f" % (j + one_based, X[i, j])
                        for j in X[i].nonzero()[is_sp]])
         f.write((u"%f %s\n" % (y[i], s)).encode('ascii'))
 
 
-def dump_svmlight_file(X, y, f):
+def dump_svmlight_file(X, y, f, zero_based=True):
     """Dump the dataset in svmlight / libsvm file format.
 
     This format is a text-based format, with one sample per line. It does
@@ -158,9 +190,15 @@ def dump_svmlight_file(X, y, f):
         Target values.
 
     f : str or file-like in binary mode
+        If string it specifies the path that will contain the data.
+        If f is a file-like then data will be written to f.
+
+    zero_based : boolean, optional
+        Whether column indices should be written zero-based (True) or one-based
+        (False).
     """
     if hasattr(f, "write"):
-        _dump_svmlight(X, y, f)
+        _dump_svmlight(X, y, f, zero_based)
     else:
         with open(f, "wb") as f:
-            _dump_svmlight(X, y, f)
+            _dump_svmlight(X, y, f, zero_based)
