@@ -9,10 +9,7 @@ from scipy import linalg
 from ..utils.arpack import eigsh
 from ..base import BaseEstimator, TransformerMixin
 from ..preprocessing import KernelCenterer
-from ..metrics.pairwise import linear_kernel
-from ..metrics.pairwise import polynomial_kernel
-from ..metrics.pairwise import rbf_kernel
-from ..metrics.pairwise import sigmoid_kernel
+from ..metrics.pairwise import pairwise_kernels
 
 
 class KernelPCA(BaseEstimator, TransformerMixin):
@@ -46,7 +43,7 @@ class KernelPCA(BaseEstimator, TransformerMixin):
         Default: 1.0
 
     fit_inverse_transform: bool
-        Learn the inverse transform.
+        Learn the inverse transform for non-precomputed kernels.
         (i.e. learn to find the pre-image of a point)
         Default: False
 
@@ -75,7 +72,8 @@ class KernelPCA(BaseEstimator, TransformerMixin):
     `X_transformed_fit_`:
         Projection of the fitted data on the kernel principal components
 
-    **References**:
+    References
+    ----------
     Kernel PCA was intoduced in:
         Bernhard Schoelkopf, Alexander J. Smola,
         and Klaus-Robert Mueller. 1999. Kernel principal
@@ -86,6 +84,9 @@ class KernelPCA(BaseEstimator, TransformerMixin):
     def __init__(self, n_components=None, kernel="linear", gamma=0, degree=3,
                  coef0=1, alpha=1.0, fit_inverse_transform=False,
                  eigen_solver='auto', tol=0, max_iter=None):
+        if fit_inverse_transform and kernel == 'precomputed':
+            raise ValueError(
+                "Cannot fit_inverse_transform with a precomputed kernel.")
         self.n_components = n_components
         self.kernel = kernel.lower()
         self.gamma = gamma
@@ -99,25 +100,13 @@ class KernelPCA(BaseEstimator, TransformerMixin):
         self.centerer = KernelCenterer()
 
     def _get_kernel(self, X, Y=None):
-        if Y is None:
-            Y = X
-
-        if self.kernel == "precomputed":
-            return X
-        elif self.kernel == "rbf":
-            return rbf_kernel(X, Y, gamma=self.gamma)
-        elif self.kernel == "poly":
-            return polynomial_kernel(X, Y,
-                                     gamma=self.gamma,
-                                     degree=self.degree,
-                                     coef0=self.coef0)
-        elif self.kernel == "sigmoid":
-            return sigmoid_kernel(X, Y,
-                                  gamma=self.gamma,
-                                  coef0=self.coef0)
-        elif self.kernel == "linear":
-            return linear_kernel(X, Y)
-        else:
+        params = {"gamma": self.gamma,
+                  "degree": self.degree,
+                  "coef0": self.coef0}
+        try:
+            return pairwise_kernels(X, Y, metric=self.kernel,
+                                    filter_params=True, **params)
+        except AttributeError:
             raise ValueError("%s is not a valid kernel. Valid kernels are: "
                              "rbf, poly, sigmoid, linear and precomputed."
                              % self.kernel)
@@ -243,7 +232,8 @@ class KernelPCA(BaseEstimator, TransformerMixin):
         -------
         X_new: array-like, shape (n_samples, n_features)
 
-        **References**:
+        References
+        ----------
         "Learning to Find Pre-Images", G BakIr et al, 2004.
         """
         if not self.fit_inverse_transform:
