@@ -1,13 +1,17 @@
 """
 Test the fastica algorithm.
 """
-
+import warnings
 import numpy as np
 from numpy.testing import assert_almost_equal
+from nose.tools import assert_true
 
 from scipy import stats
+import itertools
 
-from .. import FastICA, fastica
+from sklearn.utils.testing import assert_less
+
+from .. import FastICA, fastica, PCA
 from ..fastica_ import _gs_decorrelation
 
 
@@ -36,18 +40,18 @@ def test_gs():
     W, _, _ = np.linalg.svd(rng.randn(10, 10))
     w = rng.randn(10)
     _gs_decorrelation(w, W, 10)
-    assert (w ** 2).sum() < 1.e-10
+    assert_less((w ** 2).sum(), 1.e-10)
     w = rng.randn(10)
     u = _gs_decorrelation(w, W, 5)
     tmp = np.dot(u, W.T)
-    assert((tmp[:5] ** 2).sum() < 1.e-10)
+    assert_less((tmp[:5] ** 2).sum(), 1.e-10)
 
 
 def test_fastica(add_noise=False):
     """ Test the FastICA algorithm on very simple data.
     """
     # scipy.stats uses the global RNG:
-    np.random.seed(0)
+    rng = np.random.RandomState(0)
     n_samples = 1000
     # Generate two sources:
     s1 = (2 * np.sin(np.linspace(0, 100, n_samples)) > 0) - 1
@@ -63,35 +67,41 @@ def test_fastica(add_noise=False):
     m = np.dot(mixing, s)
 
     if add_noise:
-        m += 0.1 * np.random.randn(2, 1000)
+        m += 0.1 * rng.randn(2, 1000)
 
     center_and_norm(m)
 
-    algorithm = ['parallel', 'deflation']
-    non_linearity = ['logcosh', 'exp', 'cube']
-    for nl in non_linearity:
-        for algo in algorithm:
+    algos = ['parallel', 'deflation']
+    nls = ['logcosh', 'exp', 'cube']
+    whitening = [True, False]
+    for algo, nl, whiten in itertools.product(algos, nls, whitening):
+        if whiten:
             k_, mixing_, s_ = fastica(m.T, fun=nl, algorithm=algo)
-            s_ = s_.T
-            # Check that the mixing model described in the docstring holds:
+        else:
+            X = PCA(n_components=2, whiten=True).fit_transform(m.T)
+            k_, mixing_, s_ = fastica(X, fun=nl, algorithm=algo,
+                                     whiten=False)
+        s_ = s_.T
+        # Check that the mixing model described in the docstring holds:
+        if whiten:
             assert_almost_equal(s_, np.dot(np.dot(mixing_, k_), m))
 
-            center_and_norm(s_)
-            s1_, s2_ = s_
-            # Check to see if the sources have been estimated
-            # in the wrong order
-            if abs(np.dot(s1_, s2)) > abs(np.dot(s1_, s1)):
-                s2_, s1_ = s_
-            s1_ *= np.sign(np.dot(s1_, s1))
-            s2_ *= np.sign(np.dot(s2_, s2))
+        center_and_norm(s_)
+        s1_, s2_ = s_
+        # Check to see if the sources have been estimated
+        # in the wrong order
+        if abs(np.dot(s1_, s2)) > abs(np.dot(s1_, s1)):
+            s2_, s1_ = s_
+        s1_ *= np.sign(np.dot(s1_, s1))
+        s2_ *= np.sign(np.dot(s2_, s2))
 
-            # Check that we have estimated the original sources
-            if add_noise == False:
-                assert_almost_equal(np.dot(s1_, s1) / n_samples, 1, decimal=2)
-                assert_almost_equal(np.dot(s2_, s2) / n_samples, 1, decimal=2)
-            else:
-                assert_almost_equal(np.dot(s1_, s1) / n_samples, 1, decimal=1)
-                assert_almost_equal(np.dot(s2_, s2) / n_samples, 1, decimal=1)
+        # Check that we have estimated the original sources
+        if add_noise == False:
+            assert_almost_equal(np.dot(s1_, s1) / n_samples, 1, decimal=2)
+            assert_almost_equal(np.dot(s2_, s2) / n_samples, 1, decimal=2)
+        else:
+            assert_almost_equal(np.dot(s1_, s1) / n_samples, 1, decimal=1)
+            assert_almost_equal(np.dot(s2_, s2) / n_samples, 1, decimal=1)
 
     # Test FastICA class
     ica = FastICA(fun=nl, algorithm=algo)
@@ -105,6 +115,13 @@ def test_fastica_nowhiten():
     ica.fit(m)
     ica.get_mixing_matrix()
 
+    # test for issue #697
+    with warnings.catch_warnings(record=True) as w:
+        warnings.simplefilter("always")
+        ica = FastICA(n_components=1, whiten=False)
+        ica.fit(m)  # should raise warning
+        assert_true(len(w) == 1)  # 1 warning should be raised
+
 
 def test_non_square_fastica(add_noise=False):
     """ Test the FastICA algorithm on very simple data.
