@@ -8,3 +8,6 @@ from .forest import RandomForestClassifier
 from .forest import RandomForestRegressor
 from .forest import ExtraTreesClassifier
 from .forest import ExtraTreesRegressor
+
+from .gradient_boosting import GradientBoostingClassifier
+from .gradient_boosting import GradientBoostingRegressor
