@@ -54,9 +54,9 @@ class RFE(BaseEstimator):
         The mask of selected features.
 
     `ranking_` : array of shape [n_features]
-        The feature ranking, such that `ranking_[i]` corresponds to the ranking \
-        position of the i-th feature. Selected (i.e., estimated best) features \
-        are assigned rank 1.
+        The feature ranking, such that `ranking_[i]` corresponds to the \
+        ranking position of the i-th feature. Selected (i.e., estimated \
+        best) features are assigned rank 1.
 
     Examples
     --------
@@ -76,9 +76,8 @@ class RFE(BaseEstimator):
     >>> selector.ranking_
     array([1, 1, 1, 1, 1, 6, 4, 3, 2, 5])
 
-    Notes
-    -----
-    **References**:
+    References
+    ----------
 
     .. [1] Guyon, I., Weston, J., Barnhill, S., & Vapnik, V., "Gene selection
            for cancer classification using support vector machines",
@@ -108,7 +107,8 @@ class RFE(BaseEstimator):
             step = int(self.step * n_features)
         else:
             step = int(self.step)
-        assert step > 0
+        if step <= 0:
+            raise ValueError("Step must be >0")
 
         support_ = np.ones(n_features, dtype=np.bool)
         ranking_ = np.ones(n_features, dtype=np.int)
@@ -212,7 +212,7 @@ class RFECV(RFE):
         If int, it is the number of folds.
         If None, 3-fold cross-validation is performed by default.
         Specific cross-validation objects can also be passed, see
-        `scikits.learn.cross_validation module` for details.
+        `sklearn.cross_validation module` for details.
 
     loss_function : function, optional (default=None)
         The loss function to minimize by cross-validation. If None, then the
@@ -255,9 +255,8 @@ class RFECV(RFE):
     >>> selector.ranking_
     array([1, 1, 1, 1, 1, 6, 4, 3, 2, 5])
 
-    Notes
-    -----
-    **References**:
+    References
+    ----------
 
     .. [1] Guyon, I., Weston, J., Barnhill, S., & Vapnik, V., "Gene selection
            for cancer classification using support vector machines",
