@@ -4,12 +4,13 @@ Testing Recursive feature elimination
 
 import numpy as np
 from numpy.testing import assert_array_almost_equal
+from nose.tools import assert_true
 
-from ..rfe import RFE, RFECV
-from ...datasets import load_iris
-from ...metrics import zero_one
-from ...svm import SVC
-from ...utils import check_random_state
+from sklearn.feature_selection.rfe import RFE, RFECV
+from sklearn.datasets import load_iris
+from sklearn.metrics import zero_one
+from sklearn.svm import SVC
+from sklearn.utils import check_random_state
 
 
 def test_rfe():
@@ -24,11 +25,11 @@ def test_rfe():
     rfe.fit(X, y)
     X_r = rfe.transform(X)
 
-    assert X_r.shape == iris.data.shape
+    assert_true(X_r.shape == iris.data.shape)
     assert_array_almost_equal(X_r[:10], iris.data[:10])
 
     assert_array_almost_equal(rfe.predict(X), clf.predict(iris.data))
-    assert rfe.score(X, y) == clf.score(iris.data, iris.target)
+    assert_true(rfe.score(X, y) == clf.score(iris.data, iris.target))
 
 
 def test_rfecv():
@@ -43,7 +44,7 @@ def test_rfecv():
     rfecv.fit(X, y)
     X_r = rfecv.transform(X)
 
-    assert X_r.shape == iris.data.shape
+    assert_true(X_r.shape == iris.data.shape)
     assert_array_almost_equal(X_r[:10], iris.data[:10])
 
     # Test using a customized loss function
@@ -52,5 +53,5 @@ def test_rfecv():
     rfecv.fit(X, y)
     X_r = rfecv.transform(X)
 
-    assert X_r.shape == iris.data.shape
+    assert_true(X_r.shape == iris.data.shape)
     assert_array_almost_equal(X_r[:10], iris.data[:10])
