@@ -10,7 +10,7 @@ from scipy import linalg, optimize, rand
 
 from ..base import BaseEstimator, RegressorMixin
 from ..metrics.pairwise import manhattan_distances
-from ..utils import array2d
+from ..utils import array2d, check_random_state
 from . import regression_models as regression
 from . import correlation_models as correlation
 
@@ -62,8 +62,7 @@ def l1_cross_distances(X):
 
 
 class GaussianProcess(BaseEstimator, RegressorMixin):
-    """
-    The Gaussian Process model class.
+    """The Gaussian Process model class.
 
     Parameters
     ----------
@@ -134,9 +133,14 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
         Default is normalize = True so that data is normalized to ease
         maximum likelihood estimation.
 
-    nugget : double, optional
+    nugget : double or ndarray, optional
         Introduce a nugget effect to allow smooth predictions from noisy
-        data.
+        data.  If nugget is an ndarray, it must be the same length as the
+        number of data points used for the fit.
+        The nugget is added to the diagonal of the assumed training covariance;
+        in this way it acts as a Tikhonov regularization in the problem.  In
+        the special case of the squared exponential correlation function, the
+        nugget mathematically represents the variance of the input values.
         Default assumes a nugget close to machine precision for the sake of
         robustness (nugget = 10. * MACHINE_EPSILON).
 
@@ -159,6 +163,11 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
         exponential distribution (log-uniform on [thetaL, thetaU]).
         Default does not use random starting point (random_start = 1).
 
+    random_state: integer or numpy.RandomState, optional
+        The generator used to shuffle the sequence of coordinates of theta in
+        the Welch optimizer. If an integer is given, it fixes the seed.
+        Defaults to the global numpy random number generator.
+
     Examples
     --------
     >>> import numpy as np
@@ -166,9 +175,8 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
     >>> X = np.array([[1., 3., 5., 6., 7., 8.]]).T
     >>> y = (X * np.sin(X)).ravel()
     >>> gp = GaussianProcess(theta0=0.1, thetaL=.001, thetaU=1.)
-    >>> gp.fit(X, y) # doctest: +ELLIPSIS
-    GaussianProcess(beta0=None, corr=...,
-            normalize=..., nugget=...,
+    >>> gp.fit(X, y)                                      # doctest: +ELLIPSIS
+    GaussianProcess(beta0=None...
             ...
 
     Notes
@@ -176,15 +184,16 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
     The presentation implementation is based on a translation of the DACE
     Matlab toolbox, see reference [NLNS2002]_.
 
-    **References**:
+    References
+    ----------
 
-    .. [NLNS2002] `H.B. Nielsen, S.N. Lophaven, H. B. Nielsen and J. Sondergaard (2002).
-        DACE - A MATLAB Kriging Toolbox.`
+    .. [NLNS2002] `H.B. Nielsen, S.N. Lophaven, H. B. Nielsen and J.
+        Sondergaard.  DACE - A MATLAB Kriging Toolbox.` (2002)
         http://www2.imm.dtu.dk/~hbn/dace/dace.pdf
 
     .. [WBSWM1992] `W.J. Welch, R.J. Buck, J. Sacks, H.P. Wynn, T.J. Mitchell,
-        and M.D.  Morris (1992). Screening, predicting, and computer experiments.
-        Technometrics, 34(1) 15--25.`
+        and M.D.  Morris (1992). Screening, predicting, and computer
+        experiments.  Technometrics, 34(1) 15--25.`
         http://www.jstor.org/pss/1269548
     """
 
@@ -208,7 +217,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
                  storage_mode='full', verbose=False, theta0=1e-1,
                  thetaL=None, thetaU=None, optimizer='fmin_cobyla',
                  random_start=1, normalize=True,
-                 nugget=10. * MACHINE_EPSILON):
+                 nugget=10. * MACHINE_EPSILON, random_state=None):
 
         self.regr = regr
         self.corr = corr
@@ -222,6 +231,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
         self.nugget = nugget
         self.optimizer = optimizer
         self.random_start = random_start
+        self.random_state = random_state
 
         # Run input checks
         self._check_params()
@@ -246,9 +256,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
             A fitted Gaussian Process model object awaiting data to perform
             predictions.
         """
-
-        # Run input checks
-        self._check_params()
+        self.random_state = check_random_state(self.random_state)
 
         # Force data to 2D numpy.array
         X = array2d(np.asarray(X))
@@ -263,6 +271,9 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
         else:
             n_samples = n_samples_X
 
+        # Run input checks
+        self._check_params(n_samples)
+
         # Normalize data or don't
         if self.normalize:
             X_mean = np.mean(X, axis=0)
@@ -284,7 +295,8 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
         D, ij = l1_cross_distances(X)
         if np.min(np.sum(D, axis=1)) == 0. \
                                     and self.corr != correlation.pure_nugget:
-            raise Exception("Multiple X are not allowed")
+            raise Exception("Multiple input features cannot have the same"
+                    " value")
 
         # Regression matrix and parameters
         F = self.regr(X)
@@ -391,14 +403,14 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
             An array with shape (n_eval, ) with the Mean Squared Error at x.
         """
 
-        # Run input checks
-        self._check_params()
-
         # Check input shapes
         X = array2d(X)
         n_eval, n_features_X = X.shape
         n_samples, n_features = self.X.shape
 
+        # Run input checks
+        self._check_params(n_samples)
+
         if n_features_X != n_features:
             raise ValueError(("The number of features in X (X.shape[1] = %d) "
                            + "should match the sample size used for fit() "
@@ -743,7 +755,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
             # Iterate over all dimensions of theta allowing for anisotropy
             if verbose:
                 print("Now improving allowing for anisotropy...")
-            for i in np.random.permutation(range(theta0.size)):
+            for i in self.random_state.permutation(theta0.size):
                 if verbose:
                     print "Proceeding along dimension %d..." % (i + 1)
                 self.theta0 = array2d(theta_iso)
@@ -774,7 +786,7 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
 
         return optimal_theta, optimal_rlf_value, optimal_par
 
-    def _check_params(self):
+    def _check_params(self, n_samples=None):
 
         # Check regression model
         if not callable(self.regr):
@@ -835,8 +847,13 @@ class GaussianProcess(BaseEstimator, RegressorMixin):
         self.normalize = bool(self.normalize)
 
         # Check nugget value
-        if self.nugget < 0.:
+        self.nugget = np.asarray(self.nugget)
+        if np.any(self.nugget) < 0.:
             raise ValueError("nugget must be positive or zero.")
+        if (n_samples is not None
+            and self.nugget.shape not in [(), (n_samples,)]):
+            raise ValueError("nugget must be either a scalar "
+                             "or array of length n_samples.")
 
         # Check optimizer
         if not self.optimizer in self._optimizer_types:
