@@ -24,7 +24,7 @@ class IterGrid(object):
     """Generators on the combination of the various parameter lists given
 
     Parameters
-    -----------
+    ----------
     param_grid: dict of string to sequence
         The parameter grid to explore, as a dictionary mapping estimator
         parameters to sequences of allowed values.
@@ -36,12 +36,17 @@ class IterGrid(object):
         allowed values.
 
     Examples
-    ---------
+    --------
     >>> from sklearn.grid_search import IterGrid
     >>> param_grid = {'a':[1, 2], 'b':[True, False]}
     >>> list(IterGrid(param_grid)) #doctest: +NORMALIZE_WHITESPACE
     [{'a': 1, 'b': True}, {'a': 1, 'b': False},
      {'a': 2, 'b': True}, {'a': 2, 'b': False}]
+
+    See also
+    --------
+    :class:`GridSearchCV`:
+        uses ``IterGrid`` to perform a full parallelized grid search.
     """
 
     def __init__(self, param_grid):
@@ -90,8 +95,20 @@ def fit_grid_point(X, y, base_clf, clf_params, train, test, loss_func,
             ind = np.arange(X.shape[0])
             train = ind[train]
             test = ind[test]
-        X_train = X[train]
-        X_test = X[test]
+        if hasattr(base_clf, 'kernel') and hasattr(base_clf.kernel, '__call__'):
+            # cannot compute the kernel values with custom function
+            raise ValueError(
+                "Cannot use a custom kernel function. "
+                "Precompute the kernel matrix instead.")
+        if getattr(base_clf, 'kernel', '') == 'precomputed':
+            # X is a precomputed square kernel matrix
+            if X.shape[0] != X.shape[1]:
+                raise ValueError("X should be a square kernel matrix")
+            X_train = X[np.ix_(train, train)]
+            X_test = X[np.ix_(test, train)]
+        else:
+            X_train = X[train]
+            X_test = X[test]
     if y is not None:
         y_test = y[test]
         y_train = y[train]
@@ -127,7 +144,36 @@ def fit_grid_point(X, y, base_clf, clf_params, train, test, loss_func,
                               logger.short_format_time(time.time() -
                                                        start_time))
         print "[GridSearchCV] %s %s" % ((64 - len(end_msg)) * '.', end_msg)
-    return this_score, clf, this_n_test_samples
+    return this_score, clf_params, this_n_test_samples
+
+
+def _check_param_grid(param_grid):
+    if hasattr(param_grid, 'items'):
+        param_grid = [param_grid]
+
+    for p in param_grid:
+        for v in p.itervalues():
+            if isinstance(v, np.ndarray) and v.ndim > 1:
+                raise ValueError("Parameter array should be one-dimensional.")
+
+            check = [isinstance(v, k) for k in (list, tuple, np.ndarray)]
+            if not True in check:
+                raise ValueError("Parameter values should be a list.")
+
+            if len(v) == 0:
+                raise ValueError("Parameter values should be a non-empty list.")
+
+
+def _has_one_grid_point(param_grid):
+    if hasattr(param_grid, 'items'):
+        param_grid = [param_grid]
+
+    for p in param_grid:
+        for v in p.itervalues():
+            if len(v) > 1:
+                return False
+
+    return True
 
 
 class GridSearchCV(BaseEstimator):
@@ -192,7 +238,9 @@ class GridSearchCV(BaseEstimator):
         sklearn.cross_validation module for the list of possible objects
 
     refit: boolean
-        refit the best estimator with the entire dataset
+        refit the best estimator with the entire dataset.
+        If "False", it is impossible to make predictions using
+        this GridSearch instance after fitting.
 
     verbose: integer
         Controls the verbosity: the higher, the more messages.
@@ -209,7 +257,7 @@ class GridSearchCV(BaseEstimator):
     GridSearchCV(cv=None,
         estimator=SVC(C=1.0, cache_size=..., coef0=..., degree=...,
             gamma=..., kernel='rbf', probability=False,
-            scale_C=False, shrinking=True, tol=...),
+            shrinking=True, tol=...),
         fit_params={}, iid=True, loss_func=None, n_jobs=1,
             param_grid=...,
             ...)
@@ -227,6 +275,8 @@ class GridSearchCV(BaseEstimator):
     `best_score_` : float
         score of best_estimator on the left out data.
 
+    `best_params_` : dict
+        Parameter setting that gave the best results on the hold out data.
 
     Notes
     ------
@@ -245,7 +295,13 @@ class GridSearchCV(BaseEstimator):
 
     See Also
     ---------
-    IterGrid
+    :class:`IterGrid`:
+        generates all the combinations of a an hyperparameter grid.
+
+    :func:`sklearn.cross_validation.train_test_split`:
+        utility function to split the data into a development set usable
+        for fitting a GridSearchCV instance and an evaluation set for
+        its final evaluation.
 
     """
 
@@ -253,17 +309,21 @@ class GridSearchCV(BaseEstimator):
                  fit_params=None, n_jobs=1, iid=True, refit=True, cv=None,
                  verbose=0, pre_dispatch='2*n_jobs',
                 ):
-        assert hasattr(estimator, 'fit') and (hasattr(estimator, 'predict')
-                        or hasattr(estimator, 'score')), (
-            "estimator should a be an estimator implementing 'fit' and "
-            "'predict' or 'score' methods, %s (type %s) was passed" %
-                    (estimator, type(estimator)))
+        if not hasattr(estimator, 'fit') or \
+           not (hasattr(estimator, 'predict') or hasattr(estimator, 'score')):
+            raise TypeError("estimator should a be an estimator implementing"
+                            " 'fit' and 'predict' or 'score' methods,"
+                            " %s (type %s) was passed" %
+                            (estimator, type(estimator)))
         if loss_func is None and score_func is None:
-            assert hasattr(estimator, 'score'), ValueError(
+            if not hasattr(estimator, 'score'):
+                raise TypeError(
                     "If no loss_func is specified, the estimator passed "
                     "should have a 'score' method. The estimator %s "
                     "does not." % estimator)
 
+        _check_param_grid(param_grid)
+
         self.estimator = estimator
         self.param_grid = param_grid
         self.loss_func = loss_func
@@ -276,6 +336,12 @@ class GridSearchCV(BaseEstimator):
         self.verbose = verbose
         self.pre_dispatch = pre_dispatch
 
+    def _set_methods(self):
+        if hasattr(self._best_estimator_, 'predict'):
+            self.predict = self._best_estimator_.predict
+        if hasattr(self._best_estimator_, 'predict_proba'):
+            self.predict_proba = self._best_estimator_.predict_proba
+
     def fit(self, X, y=None, **params):
         """Run fit with all sets of parameters
 
@@ -296,6 +362,7 @@ class GridSearchCV(BaseEstimator):
         self._set_params(**params)
         estimator = self.estimator
         cv = self.cv
+
         if hasattr(X, 'shape'):
             n_samples = X.shape[0]
         else:
@@ -312,6 +379,16 @@ class GridSearchCV(BaseEstimator):
 
         grid = IterGrid(self.param_grid)
         base_clf = clone(self.estimator)
+
+        # Return early if there is only one grid point.
+        if _has_one_grid_point(self.param_grid):
+            params = iter(grid).next()
+            base_clf.set_params(**params)
+            base_clf.fit(X, y)
+            self._best_estimator_ = base_clf
+            self._set_methods()
+            return self
+
         pre_dispatch = self.pre_dispatch
         out = Parallel(n_jobs=self.n_jobs, verbose=self.verbose,
                 pre_dispatch=pre_dispatch)(
@@ -331,7 +408,7 @@ class GridSearchCV(BaseEstimator):
             n_test_samples = 0
             score = 0
             these_points = list()
-            for this_score, estimator, this_n_test_samples in \
+            for this_score, clf_params, this_n_test_samples in \
                                     out[grid_start:grid_start + n_folds]:
                 these_points.append(this_score)
                 if self.iid:
@@ -340,38 +417,31 @@ class GridSearchCV(BaseEstimator):
                 n_test_samples += this_n_test_samples
             if self.iid:
                 score /= float(n_test_samples)
-            scores.append((score, estimator))
+            scores.append((score, clf_params))
             cv_scores.append(these_points)
 
+        cv_scores = np.asarray(cv_scores)
+
         # Note: we do not use max(out) to make ties deterministic even if
         # comparison on estimator instances is not deterministic
-        best_score = None
-        for score, estimator in scores:
-            if best_score is None:
+        best_score = -np.inf
+        for score, params in scores:
+            if score > best_score:
                 best_score = score
-                best_estimator = estimator
-            else:
-                if score > best_score:
-                    best_score = score
-                    best_estimator = estimator
+                best_params = params
 
         if best_score is None:
             raise ValueError('Best score could not be found')
         self.best_score_ = best_score
+        self.best_params_ = best_params
 
         if self.refit:
             # fit the best estimator using the entire dataset
             # clone first to work around broken estimators
-            best_estimator = clone(best_estimator)
+            best_estimator = clone(base_clf).set_params(**best_params)
             best_estimator.fit(X, y, **self.fit_params)
-
-        self.best_estimator_ = best_estimator
-        if hasattr(best_estimator, 'predict'):
-            self.predict = best_estimator.predict
-        if hasattr(best_estimator, 'predict_proba'):
-            self.predict_proba = best_estimator.predict_proba
-        if hasattr(best_estimator, 'score'):
-            self.score_ = best_estimator.score
+            self._best_estimator_ = best_estimator
+            self._set_methods()
 
         # Store the computed scores
         # XXX: the name is too specific, it shouldn't have
@@ -383,11 +453,26 @@ class GridSearchCV(BaseEstimator):
         return self
 
     def score(self, X, y=None):
-        # This method is overridden during the fit if the best estimator
-        # found has a score function.
+        if hasattr(self.best_estimator_, 'score'):
+            return self.best_estimator_.score(X, y)
+        if self.score_func is None:
+            raise ValueError("No score function explicitly defined, "
+                             "and the estimator doesn't provide one %s"
+                             % self.best_estimator_)
         y_predicted = self.predict(X)
         return self.score_func(y, y_predicted)
 
+    # TODO around 0.13: remove this property, make it an attribute
+    @property
+    def best_estimator_(self):
+        if hasattr(self, '_best_estimator_'):
+            return self._best_estimator_
+        else:
+            raise RuntimeError("Grid search has to be run with 'refit=True'"
+                " to make predictions or obtain an instance  of the best "
+                " estimator. To obtain the best parameter settings, "
+                " use ``best_params_``.")
+
     @property
     @deprecated('GridSearchCV.best_estimator is deprecated'
                 ' and will be removed in version 0.12.'
