@@ -1,16 +1,21 @@
 # Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #         Fabian Pedregosa <fabian.pedregosa@inria.fr>
 #         Olivier Grisel <olivier.grisel@ensta.org>
+#         Gael Varoquaux <gael.varoquaux@inria.fr>
 #
 # License: BSD Style.
 
 import sys
 import warnings
+import itertools
+import operator
+
 import numpy as np
 
 from .base import LinearModel
 from ..utils import as_float_array
 from ..cross_validation import check_cv
+from ..externals.joblib import Parallel, delayed
 from . import cd_fast
 
 
@@ -75,6 +80,13 @@ class ElasticNet(LinearModel):
         dual gap for optimality and continues until it is smaller
         than tol.
 
+    warm_start : bool, optional
+        When set to True, reuse the solution of the previous call to fit as
+        initialization, otherwise, just erase the previous solution.
+
+    positive: bool, optional
+        When set to True, forces the coefficients to be positive.
+
     Notes
     -----
     To avoid unnecessary memory duplication the X argument of the fit method
@@ -82,7 +94,7 @@ class ElasticNet(LinearModel):
     """
     def __init__(self, alpha=1.0, rho=0.5, fit_intercept=True,
                  normalize=False, precompute='auto', max_iter=1000,
-                 copy_X=True, tol=1e-4):
+                 copy_X=True, tol=1e-4, warm_start=False, positive=False):
         self.alpha = alpha
         self.rho = rho
         self.coef_ = None
@@ -92,6 +104,8 @@ class ElasticNet(LinearModel):
         self.max_iter = max_iter
         self.copy_X = copy_X
         self.tol = tol
+        self.warm_start = warm_start
+        self.positive = positive
 
     def fit(self, X, y, Xy=None, coef_init=None):
         """Fit Elastic Net model with coordinate descent
@@ -137,7 +151,8 @@ class ElasticNet(LinearModel):
             Xy = None  # recompute Xy
 
         if coef_init is None:
-            self.coef_ = np.zeros(n_features, dtype=np.float64)
+            if not self.warm_start or self.coef_ is None:
+                self.coef_ = np.zeros(n_features, dtype=np.float64)
         else:
             self.coef_ = coef_init
 
@@ -158,14 +173,13 @@ class ElasticNet(LinearModel):
         if Gram is None:
             self.coef_, self.dual_gap_, self.eps_ = \
                     cd_fast.enet_coordinate_descent(self.coef_, alpha, beta,
-                                                    X, y, self.max_iter,
-                                                    self.tol)
+                            X, y, self.max_iter, self.tol, self.positive)
         else:
             if Xy is None:
                 Xy = np.dot(X.T, y)
             self.coef_, self.dual_gap_, self.eps_ = \
                     cd_fast.enet_coordinate_descent_gram(self.coef_, alpha,
-                                beta, Gram, Xy, y, self.max_iter, self.tol)
+                    beta, Gram, Xy, y, self.max_iter, self.tol, self.positive)
 
         self._set_intercept(X_mean, y_mean, X_std)
 
@@ -220,6 +234,13 @@ class Lasso(ElasticNet):
         dual gap for optimality and continues until it is smaller
         than tol.
 
+    warm_start : bool, optional
+        When set to True, reuse the solution of the previous call to fit as
+        initialization, otherwise, just erase the previous solution.
+
+    positive: bool, optional
+        When set to True, forces the coefficients to be positive.
+
 
     Attributes
     ----------
@@ -235,7 +256,8 @@ class Lasso(ElasticNet):
     >>> clf = linear_model.Lasso(alpha=0.1)
     >>> clf.fit([[0,0], [1, 1], [2, 2]], [0, 1, 2])
     Lasso(alpha=0.1, copy_X=True, fit_intercept=True, max_iter=1000,
-       normalize=False, precompute='auto', tol=0.0001)
+       normalize=False, positive=False, precompute='auto', tol=0.0001,
+       warm_start=False)
     >>> print clf.coef_
     [ 0.85  0.  ]
     >>> print clf.intercept_
@@ -260,11 +282,12 @@ class Lasso(ElasticNet):
 
     def __init__(self, alpha=1.0, fit_intercept=True, normalize=False,
                  precompute='auto', copy_X=True, max_iter=1000,
-                 tol=1e-4):
+                 tol=1e-4, warm_start=False, positive=False):
         super(Lasso, self).__init__(alpha=alpha, rho=1.0,
                             fit_intercept=fit_intercept, normalize=normalize,
                             precompute=precompute, copy_X=copy_X,
-                            max_iter=max_iter, tol=tol)
+                            max_iter=max_iter, tol=tol, warm_start=warm_start,
+                            positive=positive)
 
 
 ###############################################################################
@@ -330,7 +353,8 @@ def lasso_path(X, y, eps=1e-3, n_alphas=100, alphas=None,
 
     Notes
     -----
-    See examples/plot_lasso_coordinate_descent_path.py for an example.
+    See examples/linear_model/plot_lasso_coordinate_descent_path.py
+    for an example.
 
     To avoid unnecessary memory duplication the X argument of the fit method
     should be directly passed as a fortran contiguous numpy array.
@@ -475,6 +499,18 @@ def enet_path(X, y, rho=0.5, eps=1e-3, n_alphas=100, alphas=None,
     return models
 
 
+def _path_residuals(X, y, train, test, path, path_params, rho=1):
+    this_mses = list()
+    if 'rho' in path_params:
+        path_params['rho'] = rho
+    models_train = path(X[train], y[train], **path_params)
+    this_mses = np.empty(len(models_train))
+    for i_model, model in enumerate(models_train):
+        y_ = model.predict(X[test])
+        this_mses[i_model] = ((y_ - y[test]) ** 2).mean()
+    return this_mses, rho
+
+
 class LinearModelCV(LinearModel):
     """Base class for iterative model fitting along a regularization path"""
 
@@ -507,18 +543,24 @@ class LinearModelCV(LinearModel):
         y : numpy array of shape [n_samples]
             Target values
 
-        fit_params : kwargs
-            keyword arguments passed to the Lasso fit method
-
         """
         X = np.asfortranarray(X, dtype=np.float64)
         y = np.asarray(y, dtype=np.float64)
 
         # All LinearModelCV parameters except 'cv' are acceptable
-        path_params = self._get_params()
-        del path_params['cv']
+        path_params = self.get_params()
+        if 'rho' in path_params:
+            rhos = np.atleast_1d(path_params['rho'])
+            # For the first path, we need to set rho
+            path_params['rho'] = rhos[0]
+        else:
+            rhos = [1, ]
+        path_params.pop('cv', None)
+        path_params.pop('n_jobs', None)
 
         # Start to compute path on full data
+        # XXX: is this really useful: we are fitting models that we won't
+        # use later
         models = self.path(X, y, **path_params)
 
         # Update the alphas list
@@ -531,29 +573,40 @@ class LinearModelCV(LinearModel):
 
         # Compute path for all folds and compute MSE to get the best alpha
         folds = list(cv)
-        n_folds = len(folds)
-        mse_alphas = np.zeros((n_folds, n_alphas))
-        for i, (train, test) in enumerate(folds):
-            if self.verbose:
-                print '%s: fold % 2i out of % 2i' % (
-                        self.__class__.__name__, i, n_folds),
-                sys.stdout.flush()
-            models_train = self.path(X[train], y[train], **path_params)
-            for i_alpha, model in enumerate(models_train):
-                y_ = model.predict(X[test])
-                mse_alphas[i, i_alpha] += ((y_ - y[test]) ** 2).mean()
-            if self.verbose == 1:
-                print ''
+        best_mse = np.inf
+        all_mse_paths = list()
 
-        i_best_alpha = np.argmin(np.mean(mse_alphas, axis=0))
-        model = models[i_best_alpha]
+        # We do a double for loop folded in one, in order to be able to
+        # iterate in parallel on rho and folds
+        for rho, mse_alphas in itertools.groupby(
+                    Parallel(n_jobs=self.n_jobs, verbose=self.verbose)(
+                        delayed(_path_residuals)(X, y, train, test,
+                                    self.path, path_params, rho=rho)
+                            for rho in rhos for train, test in folds
+                    ), operator.itemgetter(1)):
 
+            mse_alphas = [m[0] for m in mse_alphas]
+            mse_alphas = np.array(mse_alphas)
+            mse = np.mean(mse_alphas, axis=0)
+            i_best_alpha = np.argmin(mse)
+            this_best_mse = mse[i_best_alpha]
+            all_mse_paths.append(mse_alphas.T)
+            if this_best_mse < best_mse:
+                model = models[i_best_alpha]
+                best_rho = rho
+
+        if hasattr(model, 'rho'):
+            if model.rho != best_rho:
+                # Need to refit the model
+                model.rho = best_rho
+                model.fit(X, y)
+            self.rho_ = model.rho
         self.coef_ = model.coef_
         self.intercept_ = model.intercept_
         self.alpha = model.alpha
         self.alphas = np.asarray(alphas)
         self.coef_path_ = np.asarray([model.coef_ for model in models])
-        self.mse_path_ = mse_alphas.T
+        self.mse_path_ = np.squeeze(all_mse_paths)
         return self
 
 
@@ -601,6 +654,20 @@ class LassoCV(LinearModelCV):
     verbose : bool or integer
         amount of verbosity
 
+    Attributes
+    ----------
+    `alpha_`: float
+        The amount of penalization choosen by cross validation
+
+    `coef_` : array, shape = [n_features]
+        parameter vector (w in the fomulation formula)
+
+    `intercept_` : float
+        independent term in decision function.
+
+    `mse_path_`: array, shape = [n_alphas, n_folds]
+        mean square error for the test set on each fold, varying alpha
+
     Notes
     -----
     See examples/linear_model/lasso_path_with_crossvalidation.py
@@ -618,7 +685,7 @@ class LassoCV(LinearModelCV):
     LassoLarsCV
     """
     path = staticmethod(lasso_path)
-    estimator = Lasso
+    n_jobs = 1
 
 
 class ElasticNetCV(LinearModelCV):
@@ -633,6 +700,12 @@ class ElasticNetCV(LinearModelCV):
         l1 and l2 penalties). For rho = 0
         the penalty is an L1 penalty. For rho = 1 it is an L2 penalty.
         For 0 < rho < 1, the penalty is a combination of L1 and L2
+        This parameter can be a list, in which case the different
+        values are tested by cross-validation and the one giving the best
+        prediction score is used. Note that a good choice of list of
+        values for rho is often to put more values close to 1
+        (i.e. Lasso) and less close to 0 (i.e. Ridge), as in [.1, .5, .7,
+        .9, .95, .99, 1]
 
     eps : float, optional
         Length of the path. eps=1e-3 means that
@@ -667,6 +740,30 @@ class ElasticNetCV(LinearModelCV):
     verbose : bool or integer
         amount of verbosity
 
+    n_jobs : integer, optional
+        Number of CPUs to use during the cross validation. If '-1', use
+        all the CPUs. Note that this is used only if multiple values for
+        rho are given.
+
+    Attributes
+    ----------
+    `alpha_`: float
+        The amount of penalization choosen by cross validation
+
+    `rho_`: float
+        The compromise between l1 and l2 penalization choosen by
+        cross validation
+
+    `coef_` : array, shape = [n_features]
+        parameter vector (w in the fomulation formula)
+
+    `intercept_` : float
+        independent term in decision function.
+
+    `mse_path_`: array, shape = [n_rho, n_alpha, n_folds]
+        mean square error for the test set on each fold, varying rho and
+        alpha
+
     Notes
     -----
     See examples/linear_model/lasso_path_with_crossvalidation.py
@@ -698,12 +795,11 @@ class ElasticNetCV(LinearModelCV):
 
     """
     path = staticmethod(enet_path)
-    estimator = ElasticNet
 
     def __init__(self, rho=0.5, eps=1e-3, n_alphas=100, alphas=None,
                  fit_intercept=True, normalize=False, precompute='auto',
                  max_iter=1000, tol=1e-4, cv=None, copy_X=True,
-                 verbose=0):
+                 verbose=0, n_jobs=1):
         self.rho = rho
         self.eps = eps
         self.n_alphas = n_alphas
@@ -716,3 +812,4 @@ class ElasticNetCV(LinearModelCV):
         self.cv = cv
         self.copy_X = copy_X
         self.verbose = verbose
+        self.n_jobs = n_jobs
