@@ -2,9 +2,12 @@
 Ridge regression
 """
 
-# Author: Mathieu Blondel <mathieu@mblondel.org>
+# Author:   Mathieu Blondel <mathieu@mblondel.org>
+#           Reuben Fletcher-Costin <reuben.fletchercostin@gmail.com>
 # License: Simplified BSD
 
+
+import warnings
 import numpy as np
 
 from .base import LinearModel
@@ -63,9 +66,9 @@ def ridge_regression(X, y, alpha, sample_weight=1.0, solver='auto', tol=1e-3):
     solver : {'auto', 'dense_cholesky', 'sparse_cg'}, optional
         Solver to use in the computational routines. 'delse_cholesky'
         will use the standard scipy.linalg.solve function, 'sparse_cg'
-        will use the a conjugate gradient solver as found in
+        will use the conjugate gradient solver as found in
         scipy.sparse.linalg.cg while 'auto' will chose the most
-        appropiate depending on the matrix X.
+        appropriate depending on the matrix X.
 
     tol: float
         Precision of the solution.
@@ -126,6 +129,8 @@ class Ridge(LinearModel):
     This model solves a regression model where the loss function is
     the linear least squares function and regularization is given by
     the l2-norm. Also known as Ridge Regression or Tikhonov regularization.
+    This estimator has built-in support for multi-variate regression
+    (i.e., when y is a 2d-array of shape [n_samples, n_responses]).
 
     Parameters
     ----------
@@ -198,10 +203,10 @@ class Ridge(LinearModel):
         solver : {'auto', 'dense_cholesky', 'sparse_cg'}
             Solver to use in the computational
             routines. 'delse_cholesky' will use the standard
-            scipy.linalg.solve function, 'sparse_cg' will use the a
+            scipy.linalg.solve function, 'sparse_cg' will use the
             conjugate gradient solver as found in
             scipy.sparse.linalg.cg while 'auto' will chose the most
-            appropiate depending on the matrix X.
+            appropriate depending on the matrix X.
 
         Returns
         -------
@@ -221,7 +226,7 @@ class Ridge(LinearModel):
 
 
 class RidgeClassifier(Ridge):
-    """Classifier using Ridge regression
+    """Classifier using Ridge regression.
 
     Parameters
     ----------
@@ -239,6 +244,18 @@ class RidgeClassifier(Ridge):
     normalize : boolean, optional
         If True, the regressors X are normalized
 
+    copy_X : boolean, optional, default True
+        If True, X will be copied; else, it may be overwritten.
+
+    tol: float
+        Precision of the solution.
+
+    class_weight : dict, optional
+        Weights associated with classes in the form
+        {class_label : weight}. If not given, all classes are
+        supposed to have weight one.
+
+
     Attributes
     ----------
 
@@ -252,8 +269,15 @@ class RidgeClassifier(Ridge):
     Notes
     -----
     For multi-class classification, n_class classifiers are trained in
-    a one-versus-all approach.
+    a one-versus-all approach. Concretely, this is implemented by taking
+    advantage of the multi-variate response support in Ridge.
     """
+    def __init__(self, alpha=1.0, fit_intercept=True, normalize=False,
+            copy_X=True, tol=1e-3, class_weight=None):
+        super(RidgeClassifier, self).__init__(alpha=alpha,
+                fit_intercept=fit_intercept, normalize=normalize,
+                copy_X=copy_X, tol=tol)
+        self.class_weight = class_weight
 
     def fit(self, X, y, solver='auto'):
         """Fit Ridge regression model.
@@ -269,18 +293,24 @@ class RidgeClassifier(Ridge):
         solver : {'auto', 'dense_cholesky', 'sparse_cg'}
             Solver to use in the computational
             routines. 'delse_cholesky' will use the standard
-            scipy.linalg.solve function, 'sparse_cg' will use the a
+            scipy.linalg.solve function, 'sparse_cg' will use the
             conjugate gradient solver as found in
             scipy.sparse.linalg.cg while 'auto' will chose the most
-            appropiate depending on the matrix X.
+            appropriate depending on the matrix X.
 
         Returns
         -------
         self : returns an instance of self.
         """
+        if self.class_weight is None:
+            class_weight = {}
+        else:
+            class_weight = self.class_weight
+
+        sample_weight_classes = np.array([class_weight.get(k, 1.0) for k in y])
         self.label_binarizer = LabelBinarizer()
         Y = self.label_binarizer.fit_transform(y)
-        Ridge.fit(self, X, Y, solver=solver)
+        Ridge.fit(self, X, Y, solver=solver, sample_weight=sample_weight_classes)
         return self
 
     def decision_function(self, X):
@@ -334,53 +364,88 @@ class _RidgeGCV(LinearModel):
 
     looe = y - loov = c / diag(G)
 
-    **References**:
+    References
+    ----------
     http://cbcl.mit.edu/projects/cbcl/publications/ps/MIT-CSAIL-TR-2007-025.pdf
     http://www.mit.edu/~9.520/spring07/Classes/rlsslides.pdf
     """
 
     def __init__(self, alphas=[0.1, 1.0, 10.0], fit_intercept=True,
-            normalize=False, score_func=None, loss_func=None, copy_X=True):
+            normalize=False, score_func=None, loss_func=None, copy_X=True,
+            gcv_mode=None):
         self.alphas = np.asarray(alphas)
         self.fit_intercept = fit_intercept
         self.normalize = normalize
         self.score_func = score_func
         self.loss_func = loss_func
         self.copy_X = copy_X
+        self.gcv_mode = gcv_mode
 
     def _pre_compute(self, X, y):
         # even if X is very sparse, K is usually very dense
         K = safe_sparse_dot(X, X.T, dense_output=True)
         from scipy import linalg
         v, Q = linalg.eigh(K)
-        return K, v, Q
+        QT_y = np.dot(Q.T, y)
+        return v, Q, QT_y
 
-    def _errors(self, v, Q, y, alpha):
-        G = np.dot(np.dot(Q, np.diag(1.0 / (v + alpha))), Q.T)
-        c = np.dot(G, y)
-        G_diag = np.diag(G)
-        # handle case when y is 2-d
-        G_diag = G_diag if len(y.shape) == 1 else G_diag[:, np.newaxis]
+    def _decomp_diag(self, v_prime, Q):
+        # compute diagonal of the matrix: dot(Q, dot(diag(v_prime), Q^T))
+        return (v_prime * Q ** 2).sum(axis=-1)
+
+    def _diag_dot(self, D, B):
+        # compute dot(diag(D), B)
+        if len(B.shape) > 1:
+            # handle case where B is > 1-d
+            D = D[(slice(None), ) + (np.newaxis, ) * (len(B.shape) - 1)]
+        return D * B
+
+    def _errors(self, alpha, y, v, Q, QT_y):
+        # don't construct matrix G, instead compute action on y & diagonal
+        w = 1.0 / (v + alpha)
+        c = np.dot(Q, self._diag_dot(w, QT_y))
+        G_diag = self._decomp_diag(w, Q)
+        # handle case where y is 2-d
+        if len(y.shape) != 1:
+            G_diag = G_diag[:, np.newaxis]
         return (c / G_diag) ** 2, c
 
-    def _values(self, K, v, Q, y, alpha):
-        n_samples = y.shape[0]
+    def _values(self, alpha, y, v, Q, QT_y):
+        # don't construct matrix G, instead compute action on y & diagonal
+        w = 1.0 / (v + alpha)
+        c = np.dot(Q, self._diag_dot(w, QT_y))
+        G_diag = self._decomp_diag(w, Q)
+        # handle case where y is 2-d
+        if len(y.shape) != 1:
+            G_diag = G_diag[:, np.newaxis]
+        return y - (c / G_diag), c
 
-        G = np.dot(np.dot(Q, np.diag(1.0 / (v + alpha))), Q.T)
-        c = np.dot(G, y)
-        KG = np.dot(K, G)
-        #KG = np.dot(np.dot(Q, np.diag(v / (v + alpha))), Q.T)
-        KG_diag = np.diag(KG)
+    def _pre_compute_svd(self, X, y):
+        from scipy import sparse
+        if sparse.issparse(X) and hasattr(X, 'toarray'):
+            X = X.toarray()
+        U, s, _ = np.linalg.svd(X, full_matrices=0)
+        v = s ** 2
+        UT_y = np.dot(U.T, y)
+        return v, U, UT_y
 
-        denom = np.ones(n_samples) - KG_diag
-        if len(y.shape) == 2:
+    def _errors_svd(self, alpha, y, v, U, UT_y):
+        w = ((v + alpha) ** -1) - (alpha ** -1)
+        c = np.dot(U, self._diag_dot(w, UT_y)) + (alpha ** -1) * y
+        G_diag = self._decomp_diag(w, U) + (alpha ** -1)
+        if len(y.shape) != 1:
+            # handle case where y is 2-d
+            G_diag = G_diag[:, np.newaxis]
+        return (c / G_diag) ** 2, c
+
+    def _values_svd(self, alpha, y, v, U, UT_y):
+        w = ((v + alpha) ** -1) - (alpha ** -1)
+        c = np.dot(U, self._diag_dot(w, UT_y)) + (alpha ** -1) * y
+        G_diag = self._decomp_diag(w, U) + (alpha ** -1)
+        if len(y.shape) != 1:
             # handle case when y is 2-d
-            KG_diag = KG_diag[:, np.newaxis]
-            denom = denom[:, np.newaxis]
-
-        num = np.dot(KG, y) - KG_diag * y
-
-        return num / denom, c
+            G_diag = G_diag[:, np.newaxis]
+        return y - (c / G_diag), c
 
     def fit(self, X, y, sample_weight=1.0):
         """Fit Ridge regression model
@@ -403,12 +468,38 @@ class _RidgeGCV(LinearModel):
         X = safe_asarray(X, dtype=np.float)
         y = np.asarray(y, dtype=np.float)
 
-        n_samples = X.shape[0]
+        n_samples, n_features = X.shape
 
         X, y, X_mean, y_mean, X_std = LinearModel._center_data(X, y,
                 self.fit_intercept, self.normalize, self.copy_X)
 
-        K, v, Q = self._pre_compute(X, y)
+        gcv_mode = self.gcv_mode
+        with_sw = len(np.shape(sample_weight))
+
+        if gcv_mode is None or gcv_mode == 'auto':
+            if n_features > n_samples or with_sw:
+                gcv_mode = 'eigen'
+            else:
+                gcv_mode = 'svd'
+        elif gcv_mode == "svd" and with_sw:
+            # FIXME non-uniform sample weights not yet supported
+            warnings.warn("non-uniform sample weights unsupported for svd, "
+                "forcing usage of eigen")
+            gcv_mode = 'eigen'
+
+        if gcv_mode == 'eigen':
+            _pre_compute = self._pre_compute
+            _errors = self._errors
+            _values = self._values
+        elif gcv_mode == 'svd':
+            # assert n_samples >= n_features
+            _pre_compute = self._pre_compute_svd
+            _errors = self._errors_svd
+            _values = self._values_svd
+        else:
+            raise ValueError('bad gcv_mode "%s"' % gcv_mode)
+
+        v, Q, QT_y = _pre_compute(X, y)
         n_y = 1 if len(y.shape) == 1 else y.shape[1]
         M = np.zeros((n_samples * n_y, len(self.alphas)))
         C = []
@@ -417,9 +508,9 @@ class _RidgeGCV(LinearModel):
 
         for i, alpha in enumerate(self.alphas):
             if error:
-                out, c = self._errors(v, Q, y, sample_weight * alpha)
+                out, c = _errors(sample_weight * alpha, y, v, Q, QT_y)
             else:
-                out, c = self._values(K, v, Q, y, sample_weight * alpha)
+                out, c = _values(sample_weight * alpha, y, v, Q, QT_y)
             M[:, i] = out.ravel()
             C.append(c)
 
@@ -443,8 +534,7 @@ class RidgeCV(LinearModel):
     """Ridge regression with built-in cross-validation.
 
     By default, it performs Generalized Cross-Validation, which is a form of
-    efficient Leave-One-Out cross-validation. Currently, only the n_features >
-    n_samples case is handled efficiently.
+    efficient Leave-One-Out cross-validation.
 
     Parameters
     ----------
@@ -452,7 +542,7 @@ class RidgeCV(LinearModel):
         Array of alpha values to try.
         Small positive values of alpha improve the conditioning of the
         problem and reduce the variance of the estimates.
-        Alpha corresponds to (2*C)^-1 in other linear models such as
+        Alpha corresponds to ``(2*C)^-1`` in other linear models such as
         LogisticRegression or LinearSVC.
 
     fit_intercept : boolean
@@ -463,29 +553,54 @@ class RidgeCV(LinearModel):
     normalize : boolean, optional
         If True, the regressors X are normalized
 
-    loss_func: callable, optional
-        function that takes 2 arguments and compares them in
-        order to evaluate the performance of prediciton (small is good)
-        if None is passed, the score of the estimator is maximized
-
     score_func: callable, optional
         function that takes 2 arguments and compares them in
-        order to evaluate the performance of prediciton (big is good)
+        order to evaluate the performance of prediction (big is good)
         if None is passed, the score of the estimator is maximized
 
+    loss_func: callable, optional
+        function that takes 2 arguments and compares them in
+        order to evaluate the performance of prediction (small is good)
+        if None is passed, the score of the estimator is maximized
+
+    cv : cross-validation generator, optional
+        If None, Generalized Cross-Validation (efficient Leave-One-Out)
+        will be used.
+
+
+    Attributes
+    ----------
+    `coef_` : array, shape = [n_features] or [n_classes, n_features]
+        Weight vector(s).
+
+    gcv_mode : {None, 'auto', 'svd', eigen'}, optional
+        Flag indicating which strategy to use when performing
+        Generalized Cross-Validation. Options are::
+
+            'auto' : use svd if n_samples > n_features, otherwise use eigen
+            'svd' : force computation via singular value decomposition of X
+            'eigen' : force computation via eigendecomposition of X^T X
+
+        The 'auto' mode is the default and is intended to pick the cheaper \
+        option of the two depending upon the shape of the training data.
+
     See also
     --------
-    Ridge, RidgeClassifierCV
+    Ridge: Ridge regression
+    RidgeClassifier: Ridge classifier
+    RidgeCV: Ridge regression with built-in cross validation
     """
 
     def __init__(self, alphas=np.array([0.1, 1.0, 10.0]), fit_intercept=True,
-                   normalize=False, score_func=None, loss_func=None, cv=None):
+            normalize=False, score_func=None, loss_func=None, cv=None,
+            gcv_mode=None):
         self.alphas = alphas
         self.fit_intercept = fit_intercept
         self.normalize = normalize
         self.score_func = score_func
         self.loss_func = loss_func
         self.cv = cv
+        self.gcv_mode = gcv_mode
 
     def fit(self, X, y, sample_weight=1.0):
         """Fit Ridge regression model
@@ -501,17 +616,13 @@ class RidgeCV(LinearModel):
         sample_weight : float or array-like of shape [n_samples]
             Sample weight
 
-        cv : cross-validation generator, optional
-            If None, Generalized Cross-Validationn (efficient Leave-One-Out)
-            will be used.
-
         Returns
         -------
         self : Returns self.
         """
         if self.cv is None:
             estimator = _RidgeGCV(self.alphas, self.fit_intercept,
-                                  self.score_func, self.loss_func)
+                    self.score_func, self.loss_func, gcv_mode=self.gcv_mode)
             estimator.fit(X, y, sample_weight=sample_weight)
             self.best_alpha = estimator.best_alpha
         else:
@@ -533,6 +644,67 @@ class RidgeCV(LinearModel):
 
 
 class RidgeClassifierCV(RidgeCV):
+    """Ridge classifier with built-in cross-validation.
+
+    By default, it performs Generalized Cross-Validation, which is a form of
+    efficient Leave-One-Out cross-validation. Currently, only the n_features >
+    n_samples case is handled efficiently.
+
+    Parameters
+    ----------
+    alphas: numpy array of shape [n_alpha]
+        Array of alpha values to try.
+        Small positive values of alpha improve the conditioning of the
+        problem and reduce the variance of the estimates.
+        Alpha corresponds to (2*C)^-1 in other linear models such as
+        LogisticRegression or LinearSVC.
+
+    fit_intercept : boolean
+        Whether to calculate the intercept for this model. If set
+        to false, no intercept will be used in calculations
+        (e.g. data is expected to be already centered).
+
+    normalize : boolean, optional
+        If True, the regressors X are normalized
+
+    score_func: callable, optional
+        function that takes 2 arguments and compares them in
+        order to evaluate the performance of prediction (big is good)
+        if None is passed, the score of the estimator is maximized
+
+    loss_func: callable, optional
+        function that takes 2 arguments and compares them in
+        order to evaluate the performance of prediction (small is good)
+        if None is passed, the score of the estimator is maximized
+
+    cv : cross-validation generator, optional
+        If None, Generalized Cross-Validation (efficient Leave-One-Out)
+        will be used.
+
+    class_weight : dict, optional
+        Weights associated with classes in the form
+        {class_label : weight}. If not given, all classes are
+        supposed to have weight one.
+
+    See also
+    --------
+    Ridge: Ridge regression
+    RidgeClassifier: Ridge classifier
+    RidgeCV: Ridge regression with built-in cross validation
+
+    Notes
+    -----
+    For multi-class classification, n_class classifiers are trained in
+    a one-versus-all approach. Concretely, this is implemented by taking
+    advantage of the multi-variate response support in Ridge.
+    """
+    def __init__(self, alphas=np.array([0.1, 1.0, 10.0]), fit_intercept=True,
+            normalize=False, score_func=None, loss_func=None, cv=None,
+            class_weight=None):
+        super(RidgeClassifierCV, self).__init__(alphas=alphas,
+                fit_intercept=fit_intercept, normalize=normalize,
+                score_func=score_func, loss_func=loss_func, cv=cv)
+        self.class_weight = class_weight
 
     def fit(self, X, y, sample_weight=1.0, class_weight=None):
         """Fit the ridge classifier.
@@ -546,26 +718,34 @@ class RidgeClassifierCV(RidgeCV):
         y : array-like, shape = [n_samples]
             Target values.
 
+        sample_weight : float or numpy array of shape [n_samples]
+            Sample weight
+
         class_weight : dict, optional
             Weights associated with classes in the form
             {class_label : weight}. If not given, all classes are
             supposed to have weight one.
 
-        sample_weight : float or numpy array of shape [n_samples]
-            Sample weight
-
         Returns
         -------
         self : object
             Returns self.
         """
-        if class_weight is None:
-            class_weight = {}
-        sample_weight2 = np.array([class_weight.get(k, 1.0) for k in y])
+        if class_weight != None:
+            warnings.warn("'class_weight' is now an initialization parameter."
+                    "Using it in the 'fit' method is deprecated.",
+                    DeprecationWarning)
+            self.class_weight_ = class_weight
+        else:
+            self.class_weight_ = self.class_weight
+
+        if self.class_weight_ is None:
+            self.class_weight_ = {}
+
+        sample_weight2 = np.array([self.class_weight_.get(k, 1.0) for k in y])
         self.label_binarizer = LabelBinarizer()
         Y = self.label_binarizer.fit_transform(y)
-        RidgeCV.fit(self, X, Y,
-                    sample_weight=sample_weight * sample_weight2)
+        RidgeCV.fit(self, X, Y, sample_weight=sample_weight * sample_weight2)
         return self
 
     def decision_function(self, X):
