@@ -1,4 +1,6 @@
+import os
 from os.path import join
+
 import numpy
 
 
@@ -15,6 +17,10 @@ def configuration(parent_package='', top_path=None):
         blas_info.pop('libraries', None)
     else:
         cblas_libs = blas_info.pop('libraries', [])
+    libraries = []
+    if os.name == 'posix':
+        cblas_libs.append('m')
+        libraries.append('m')
 
     config.add_extension('cd_fast',
          sources=['cd_fast.c'],
@@ -28,11 +34,8 @@ def configuration(parent_package='', top_path=None):
 
     config.add_extension('sgd_fast',
          sources=['sgd_fast.c'],
-         include_dirs=[numpy.get_include()]
-         )
-    config.add_extension('sgd_fast_sparse',
-         sources=['sgd_fast_sparse.c'],
-         include_dirs=[numpy.get_include()]
+         include_dirs=[numpy.get_include()],
+         libraries=libraries,
          )
 
     # add other directories
