@@ -4,6 +4,8 @@ from numpy.testing import assert_array_almost_equal
 from numpy.testing import assert_almost_equal
 from numpy.testing import assert_equal
 
+from sklearn.utils.testing import assert_less, assert_greater
+
 from sklearn.linear_model.sparse.coordinate_descent import Lasso as SparseLasso
 from sklearn.linear_model.sparse.coordinate_descent \
         import ElasticNet as SparseENet
@@ -137,19 +139,19 @@ def test_sparse_enet_not_as_toy_dataset():
                        max_iter=max_iter, tol=1e-7)
     s_clf.fit(X_train, y_train)
     assert_almost_equal(s_clf.dual_gap_, 0, 4)
-    assert s_clf.score(X_test, y_test) > 0.85
+    assert_greater(s_clf.score(X_test, y_test), 0.85)
 
     # check the convergence is the same as the dense version
     d_clf = DenseENet(alpha=0.1, rho=0.8, fit_intercept=False,
                       max_iter=max_iter, tol=1e-7)
     d_clf.fit(X_train, y_train)
     assert_almost_equal(d_clf.dual_gap_, 0, 4)
-    assert d_clf.score(X_test, y_test) > 0.85
+    assert_greater(d_clf.score(X_test, y_test), 0.85)
 
     assert_almost_equal(s_clf.coef_, d_clf.coef_, 5)
 
     # check that the coefs are sparse
-    assert np.sum(s_clf.coef_ != 0.0) < 2 * n_informative
+    assert_less(np.sum(s_clf.coef_ != 0.0), 2 * n_informative)
 
 
 def test_sparse_lasso_not_as_toy_dataset():
@@ -165,14 +167,14 @@ def test_sparse_lasso_not_as_toy_dataset():
                         max_iter=max_iter, tol=1e-7)
     s_clf.fit(X_train, y_train)
     assert_almost_equal(s_clf.dual_gap_, 0, 4)
-    assert s_clf.score(X_test, y_test) > 0.85
+    assert_greater(s_clf.score(X_test, y_test), 0.85)
 
     # check the convergence is the same as the dense version
     d_clf = DenseLasso(alpha=0.1, fit_intercept=False, max_iter=max_iter,
             tol=1e-7)
     d_clf.fit(X_train, y_train)
     assert_almost_equal(d_clf.dual_gap_, 0, 4)
-    assert d_clf.score(X_test, y_test) > 0.85
+    assert_greater(d_clf.score(X_test, y_test), 0.85)
 
     # check that the coefs are sparse
     assert_equal(np.sum(s_clf.coef_ != 0.0), n_informative)
