@@ -1,13 +1,17 @@
 import numpy as np
+import scipy.sparse as sp
 
 from numpy.testing import assert_array_equal
 import nose
 from nose.tools import assert_raises, raises
 
+from sklearn.utils.testing import assert_greater
+
 from sklearn.linear_model import logistic
 from sklearn import datasets
 
 X = [[-1, 0], [0, 1], [1, 1]]
+X_sp = sp.csr_matrix(X)
 Y1 = [0, 1, 1]
 Y2 = [2, 1, 0]
 iris = datasets.load_iris()
@@ -22,14 +26,26 @@ def test_predict_2_classes():
     assert_array_equal(clf.predict(X), Y1)
     assert_array_equal(clf.predict_proba(X).argmax(axis=1), Y1)
 
+    clf = logistic.LogisticRegression().fit(X_sp, Y1)
+    assert_array_equal(clf.predict(X_sp), Y1)
+    assert_array_equal(clf.predict_proba(X_sp).argmax(axis=1), Y1)
+
     clf = logistic.LogisticRegression(C=100).fit(X, Y1)
     assert_array_equal(clf.predict(X), Y1)
     assert_array_equal(clf.predict_proba(X).argmax(axis=1), Y1)
 
+    clf = logistic.LogisticRegression(C=100).fit(X_sp, Y1)
+    assert_array_equal(clf.predict(X_sp), Y1)
+    assert_array_equal(clf.predict_proba(X_sp).argmax(axis=1), Y1)
+
     clf = logistic.LogisticRegression(fit_intercept=False).fit(X, Y1)
     assert_array_equal(clf.predict(X), Y1)
     assert_array_equal(clf.predict_proba(X).argmax(axis=1), Y1)
 
+    clf = logistic.LogisticRegression(fit_intercept=False).fit(X_sp, Y1)
+    assert_array_equal(clf.predict(X_sp), Y1)
+    assert_array_equal(clf.predict_proba(X_sp).argmax(axis=1), Y1)
+
 
 def test_error():
     """Test for appropriate exception on errors"""
@@ -41,22 +57,28 @@ def test_predict_3_classes():
     assert_array_equal(clf.predict(X), Y2)
     assert_array_equal(clf.predict_proba(X).argmax(axis=1), Y2)
 
+    clf = logistic.LogisticRegression(C=10).fit(X_sp, Y2)
+    assert_array_equal(clf.predict(X_sp), Y2)
+    assert_array_equal(clf.predict_proba(X_sp).argmax(axis=1), Y2)
+
 
 def test_predict_iris():
     """Test logisic regression with the iris dataset"""
 
-    clf = logistic.LogisticRegression().fit(iris.data, iris.target)
+    clf = logistic.LogisticRegression(C=len(iris.data)).fit(iris.data,
+                                                            iris.target)
 
     pred = clf.predict(iris.data)
-    assert np.mean(pred == iris.target) > .95
+    assert_greater(np.mean(pred == iris.target), .95)
 
     pred = clf.predict_proba(iris.data).argmax(axis=1)
-    assert np.mean(pred == iris.target) > .95
+    assert_greater(np.mean(pred == iris.target), .95)
 
 
 def test_inconsistent_input():
     """Test that an exception is raised on inconsistent input"""
-    X_ = np.random.random((5, 10))
+    rng = np.random.RandomState(0)
+    X_ = rng.random_sample((5, 10))
     y_ = np.ones(X_.shape[0])
 
     clf = logistic.LogisticRegression()
@@ -66,9 +88,8 @@ def test_inconsistent_input():
     assert_raises(ValueError, clf.fit, X, y_wrong)
 
     # Wrong dimensions for test data
-    assert_raises(ValueError,
-                  clf.fit(X_, y_).predict,
-                  np.random.random((3, 12)))
+    assert_raises(ValueError, clf.fit(X_, y_).predict,
+            rng.random_sample((3, 12)))
 
 
 @raises(ValueError)
@@ -82,15 +103,5 @@ def test_nan():
     logistic.LogisticRegression().fit(Xnan, Y1)
 
 
-def test_transform():
-    clf = logistic.LogisticRegression(penalty="l1")
-    clf.fit(iris.data, iris.target)
-    X_new = clf.transform(iris.data)
-    clf = logistic.LogisticRegression()
-    clf.fit(X_new, iris.target)
-    pred = clf.predict(X_new)
-    assert np.mean(pred == iris.target) >= 0.75
-
-
 if __name__ == '__main__':
     nose.runmodule()
