@@ -1,13 +1,13 @@
 import numpy as np
 import scipy.sparse as sp
-
+from nose.tools import assert_true
 from numpy.testing import assert_almost_equal, assert_array_almost_equal, \
-                          assert_equal
+                          assert_equal, assert_array_equal
 from sklearn import datasets
-from sklearn.metrics import mean_square_error
+from sklearn.metrics import mean_squared_error
+from sklearn.utils.testing import assert_greater
 
 from sklearn.linear_model.base import LinearRegression
-
 from sklearn.linear_model.ridge import Ridge
 from sklearn.linear_model.ridge import _RidgeGCV
 from sklearn.linear_model.ridge import RidgeCV
@@ -17,11 +17,11 @@ from sklearn.linear_model.ridge import RidgeClassifierCV
 
 from sklearn.cross_validation import KFold
 
+rng = np.random.RandomState(0)
 diabetes = datasets.load_diabetes()
-
 X_diabetes, y_diabetes = diabetes.data, diabetes.target
 ind = np.arange(X_diabetes.shape[0])
-np.random.shuffle(ind)
+rng.shuffle(ind)
 ind = ind[:200]
 X_diabetes, y_diabetes = X_diabetes[ind], y_diabetes[ind]
 
@@ -30,8 +30,6 @@ iris = datasets.load_iris()
 X_iris = sp.csr_matrix(iris.data)
 y_iris = iris.target
 
-np.random.seed(0)
-
 DENSE_FILTER = lambda X: X
 SPARSE_FILTER = lambda X: sp.csr_matrix(X)
 
@@ -46,27 +44,69 @@ def test_ridge():
 
     # With more samples than features
     n_samples, n_features = 6, 5
-    y = np.random.randn(n_samples)
-    X = np.random.randn(n_samples, n_features)
+    y = rng.randn(n_samples)
+    X = rng.randn(n_samples, n_features)
 
     ridge = Ridge(alpha=alpha)
     ridge.fit(X, y)
     assert_equal(ridge.coef_.shape, (X.shape[1], ))
-    assert ridge.score(X, y) > 0.5
+    assert_greater(ridge.score(X, y), 0.5)
 
     ridge.fit(X, y, sample_weight=np.ones(n_samples))
-    assert ridge.score(X, y) > 0.5
+    assert_greater(ridge.score(X, y), 0.5)
 
     # With more features than samples
     n_samples, n_features = 5, 10
-    y = np.random.randn(n_samples)
-    X = np.random.randn(n_samples, n_features)
+    y = rng.randn(n_samples)
+    X = rng.randn(n_samples, n_features)
     ridge = Ridge(alpha=alpha)
     ridge.fit(X, y)
-    assert ridge.score(X, y) > .9
+    assert_greater(ridge.score(X, y), .9)
 
     ridge.fit(X, y, sample_weight=np.ones(n_samples))
-    assert ridge.score(X, y) > 0.9
+    assert_greater(ridge.score(X, y), 0.9)
+
+
+def test_ridge_shapes():
+    """Test shape of coef_ and intercept_
+    """
+    n_samples, n_features = 5, 10
+    X = rng.randn(n_samples, n_features)
+    y = rng.randn(n_samples)
+    Y1 = y[:, np.newaxis]
+    Y = np.c_[y, 1 + y]
+
+    ridge = Ridge()
+
+    ridge.fit(X, y)
+    assert_equal(ridge.coef_.shape, (n_features,))
+    assert_equal(ridge.intercept_.shape, ())
+
+    ridge.fit(X, Y1)
+    assert_equal(ridge.coef_.shape, (1, n_features))
+    assert_equal(ridge.intercept_.shape, (1, ))
+
+    ridge.fit(X, Y)
+    assert_equal(ridge.coef_.shape, (2, n_features))
+    assert_equal(ridge.intercept_.shape, (2, ))
+
+
+def test_ridge_intercept():
+    """Test intercept with multiple targets GH issue #708
+    """
+    n_samples, n_features = 5, 10
+    X = rng.randn(n_samples, n_features)
+    y = rng.randn(n_samples)
+    Y = np.c_[y, 1. + y]
+
+    ridge = Ridge()
+
+    ridge.fit(X, y)
+    intercept = ridge.intercept_
+
+    ridge.fit(X, Y)
+    assert_almost_equal(ridge.intercept_[0], intercept)
+    assert_almost_equal(ridge.intercept_[1], intercept + 1.)
 
 
 def test_toy_ridge_object():
@@ -98,9 +138,8 @@ def test_ridge_vs_lstsq():
 
     # we need more samples than features
     n_samples, n_features = 5, 4
-    np.random.seed(0)
-    y = np.random.randn(n_samples)
-    X = np.random.randn(n_samples, n_features)
+    y = rng.randn(n_samples)
+    X = rng.randn(n_samples, n_features)
 
     ridge = Ridge(alpha=0., fit_intercept=False)
     ols = LinearRegression(fit_intercept=False)
@@ -124,9 +163,9 @@ def _test_ridge_loo(filter_):
     ridge = Ridge(fit_intercept=False)
 
     # generalized cross-validation (efficient leave-one-out)
-    K, v, Q = ridge_gcv._pre_compute(X_diabetes, y_diabetes)
-    errors, c = ridge_gcv._errors(v, Q, y_diabetes, 1.0)
-    values, c = ridge_gcv._values(K, v, Q, y_diabetes, 1.0)
+    decomp = ridge_gcv._pre_compute(X_diabetes, y_diabetes)
+    errors, c = ridge_gcv._errors(1.0, y_diabetes, *decomp)
+    values, c = ridge_gcv._values(1.0, y_diabetes, *decomp)
 
     # brute-force leave-one-out: remove one example at a time
     errors2 = []
@@ -145,13 +184,23 @@ def _test_ridge_loo(filter_):
     assert_almost_equal(errors, errors2)
     assert_almost_equal(values, values2)
 
+    # generalized cross-validation (efficient leave-one-out,
+    # SVD variation)
+    decomp = ridge_gcv._pre_compute_svd(X_diabetes, y_diabetes)
+    errors3, c = ridge_gcv._errors_svd(1.0, y_diabetes, *decomp)
+    values3, c = ridge_gcv._values_svd(1.0, y_diabetes, *decomp)
+
+    # check that efficient and SVD efficient LOO give same results
+    assert_almost_equal(errors, errors3)
+    assert_almost_equal(values, values3)
+
     # check best alpha
     ridge_gcv.fit(filter_(X_diabetes), y_diabetes)
     best_alpha = ridge_gcv.best_alpha
     ret.append(best_alpha)
 
     # check that we get same best alpha with custom loss_func
-    ridge_gcv2 = _RidgeGCV(fit_intercept=False, loss_func=mean_square_error)
+    ridge_gcv2 = _RidgeGCV(fit_intercept=False, loss_func=mean_squared_error)
     ridge_gcv2.fit(filter_(X_diabetes), y_diabetes)
     assert_equal(ridge_gcv2.best_alpha, best_alpha)
 
@@ -221,14 +270,14 @@ def _test_ridge_classifiers(filter_):
         clf.fit(filter_(X_iris), y_iris)
         assert_equal(clf.coef_.shape, (n_classes, n_features))
         y_pred = clf.predict(filter_(X_iris))
-        assert np.mean(y_iris == y_pred) >= 0.8
+        assert_true(np.mean(y_iris == y_pred) >= 0.8)
 
     n_samples = X_iris.shape[0]
     cv = KFold(n_samples, 5)
     clf = RidgeClassifierCV(cv=cv)
     clf.fit(filter_(X_iris), y_iris)
     y_pred = clf.predict(filter_(X_iris))
-    assert np.mean(y_iris == y_pred) >= 0.8
+    assert_true(np.mean(y_iris == y_pred) >= 0.8)
 
 
 def _test_tolerance(filter_):
@@ -240,7 +289,7 @@ def _test_tolerance(filter_):
     ridge2.fit(filter_(X_diabetes), y_diabetes)
     score2 = ridge2.score(filter_(X_diabetes), y_diabetes)
 
-    assert score >= score2
+    assert_true(score >= score2)
 
 
 def test_dense_sparse():
@@ -257,3 +306,42 @@ def test_dense_sparse():
         # test that the outputs are the same
         if ret_dense != None and ret_sparse != None:
             assert_array_almost_equal(ret_dense, ret_sparse, decimal=3)
+
+
+def test_class_weights():
+    """
+    Test class weights.
+    """
+    X = np.array([[-1.0, -1.0], [-1.0, 0], [-.8, -1.0],
+                  [1.0, 1.0], [1.0, 0.0]])
+    y = [1, 1, 1, -1, -1]
+
+    clf = RidgeClassifier(class_weight=None)
+    clf.fit(X, y)
+    assert_array_equal(clf.predict([[0.2, -1.0]]), np.array([1]))
+
+    # we give a small weights to class 1
+    clf = RidgeClassifier(class_weight={1: 0.001})
+    clf.fit(X, y)
+
+    # now the hyperplane should rotate clock-wise and
+    # the prediction on this point should shift
+    assert_array_equal(clf.predict([[0.2, -1.0]]), np.array([-1]))
+
+
+def test_class_weights_cv():
+    """
+    Test class weights for cross validated ridge classifier.
+    """
+    X = np.array([[-1.0, -1.0], [-1.0, 0], [-.8, -1.0],
+                  [1.0, 1.0], [1.0, 0.0]])
+    y = [1, 1, 1, -1, -1]
+
+    clf = RidgeClassifierCV(class_weight=None, alphas=[.01, .1, 1])
+    clf.fit(X, y)
+
+    # we give a small weights to class 1
+    clf = RidgeClassifierCV(class_weight={1: 0.001}, alphas=[.01, .1, 1, 10])
+    clf.fit(X, y)
+
+    assert_array_equal(clf.predict([[-.2, 2]]), np.array([-1]))
