@@ -5,10 +5,11 @@
 # License: BSD, (C) INRIA 2011
 
 import numpy as np
+import warnings
 from scipy.linalg import eigh, svd, qr, solve
 from scipy.sparse import eye, csr_matrix
 from ..base import BaseEstimator
-from ..utils import array2d
+from ..utils import array2d, check_random_state
 from ..utils.arpack import eigsh
 from ..neighbors import NearestNeighbors
 
@@ -102,7 +103,8 @@ def barycenter_kneighbors_graph(X, n_neighbors, reg=1e-3):
                       shape=(n_samples, n_samples))
 
 
-def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100):
+def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100,
+               random_state=None):
     """
     Find the null space of a matrix M.
 
@@ -133,6 +135,10 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100):
 
     max_iter : maximum number of iterations for 'arpack' method
         not used if eigen_solver=='dense'
+
+    random_state: numpy.RandomState, optional
+        The generator used to initialize the centers. Defaults to numpy.random.
+        Used to determine the starting vector for arpack iterations
     """
     if eigen_solver == 'auto':
         if M.shape[0] > 200 and k + k_skip < 10:
@@ -141,9 +147,12 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100):
             eigen_solver = 'dense'
 
     if eigen_solver == 'arpack':
+        random_state = check_random_state(random_state)
+        v0 = random_state.rand(M.shape[0])
         try:
             eigen_values, eigen_vectors = eigsh(M, k + k_skip, sigma=0.0,
-                                                tol=tol, maxiter=max_iter)
+                                                tol=tol, maxiter=max_iter,
+                                                v0=v0)
         except RuntimeError as msg:
             raise ValueError("Error in determining null-space with ARPACK. "
                              "Error message: '%s'. "
@@ -169,9 +178,10 @@ def null_space(M, k, k_skip=1, eigen_solver='arpack', tol=1E-6, max_iter=100):
 
 
 def locally_linear_embedding(
-    X, n_neighbors, out_dim, reg=1e-3, eigen_solver='auto',
+    X, n_neighbors, n_components, reg=1e-3, eigen_solver='auto',
     tol=1e-6, max_iter=100, method='standard',
-    hessian_tol=1E-4, modified_tol=1E-12):
+    hessian_tol=1E-4, modified_tol=1E-12,
+    random_state=None, out_dim=None):
     """Perform a Locally Linear Embedding analysis on the data.
 
     Parameters
@@ -184,7 +194,7 @@ def locally_linear_embedding(
     n_neighbors : integer
         number of neighbors to consider for each point.
 
-    out_dim : integer
+    n_components : integer
         number of coordinates for the manifold.
 
     reg : float
@@ -212,14 +222,14 @@ def locally_linear_embedding(
 
     method : {'standard', 'hessian', 'modified', 'ltsa'}
         standard : use the standard locally linear embedding algorithm.
-                   see reference [Roweis2000]_
+                   see reference [1]_
         hessian  : use the Hessian eigenmap method.  This method requires
-                   n_neighbors > out_dim * (1 + (out_dim + 1) / 2.
-                   see reference [Donoho2003]_
+                   n_neighbors > n_components * (1 + (n_components + 1) / 2.
+                   see reference [2]_
         modified : use the modified locally linear embedding algorithm.
-                   see reference [Zhang2007]_
+                   see reference [3]_
         ltsa     : use local tangent space alignment algorithm
-                   see reference [Zhang2004]_
+                   see reference [4]_
 
     hessian_tol : float, optional
         Tolerance for Hessian eigenmapping method.
@@ -229,30 +239,32 @@ def locally_linear_embedding(
         Tolerance for modified LLE method.
         Only used if method == 'modified'
 
+    random_state: numpy.RandomState, optional
+        The generator used to initialize the centers. Defaults to numpy.random.
+
     Returns
     -------
-    Y : array-like, shape [n_samples, out_dim]
+    Y : array-like, shape [n_samples, n_components]
         Embedding vectors.
 
     squared_error : float
         Reconstruction error for the embedding vectors. Equivalent to
         ``norm(Y - W Y, 'fro')**2``, where W are the reconstruction weights.
 
-    Notes
-    -----
-    **References**:
+    References
+    ----------
 
-      .. [Roweis2000] `Roweis, S. & Saul, L. Nonlinear dimensionality reduction by
-          locally linear embedding.  Science 290:2323 (2000).`
-      .. [Donoho2003] `Donoho, D. & Grimes, C. Hessian eigenmaps: Locally linear embedding
-          techniques for high-dimensional data. Proc Natl Acad Sci U S A.
-          100:5591 (2003).`
-      .. [Zhang2007] `Zhang, Z. & Wang, J. MLLE: Modified Locally Linear Embedding
-          Using Multiple Weights.`
-          http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.70.382
-      .. [Zhang2004] `Zhang, Z. & Zha, H. Principal manifolds and nonlinear dimensionality
-          reduction via tangent space alignment. Journal of Shanghai Univ.
-          8:406 (2004)`
+    .. [1] `Roweis, S. & Saul, L. Nonlinear dimensionality reduction
+        by locally linear embedding.  Science 290:2323 (2000).`
+    .. [2] `Donoho, D. & Grimes, C. Hessian eigenmaps: Locally
+        linear embedding techniques for high-dimensional data.
+        Proc Natl Acad Sci U S A.  100:5591 (2003).`
+    .. [3] `Zhang, Z. & Wang, J. MLLE: Modified Locally Linear
+        Embedding Using Multiple Weights.`
+        http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.70.382
+    .. [4] `Zhang, Z. & Zha, H. Principal manifolds and nonlinear
+        dimensionality reduction via tangent space alignment.
+        Journal of Shanghai Univ.  8:406 (2004)`
     """
     if eigen_solver not in ('auto', 'arpack', 'dense'):
         raise ValueError("unrecognized eigen_solver '%s'" % eigen_solver)
@@ -260,13 +272,19 @@ def locally_linear_embedding(
     if method not in ('standard', 'hessian', 'modified', 'ltsa'):
         raise ValueError("unrecognized method '%s'" % method)
 
+    if out_dim:
+        warnings.warn("Parameter ``out_dim`` was renamed to ``n_components`` "
+                "and is now deprecated.", DeprecationWarning,
+                stacklevel=2)
+        n_components = out_dim
+
     nbrs = NearestNeighbors(n_neighbors=n_neighbors + 1)
     nbrs.fit(X)
     X = nbrs._fit_X
 
     N, d_in = X.shape
 
-    if out_dim > d_in:
+    if n_components > d_in:
         raise ValueError("output dimension must be less than or equal "
                          "to input dimension")
     if n_neighbors >= N:
@@ -291,17 +309,17 @@ def locally_linear_embedding(
             M.flat[::M.shape[0] + 1] += 1  # W = W - I = W - I
 
     elif method == 'hessian':
-        dp = out_dim * (out_dim + 1) / 2
+        dp = n_components * (n_components + 1) / 2
 
-        if n_neighbors <= out_dim + dp:
+        if n_neighbors <= n_components + dp:
             raise ValueError("for method='hessian', n_neighbors must be "
-                             "greater than [out_dim * (out_dim + 3) / 2]")
+                    "greater than [n_components * (n_components + 3) / 2]")
 
         neighbors = nbrs.kneighbors(X, n_neighbors=n_neighbors + 1,
                                     return_distance=False)
         neighbors = neighbors[:, 1:]
 
-        Yi = np.empty((n_neighbors, 1 + out_dim + dp), dtype=np.float)
+        Yi = np.empty((n_neighbors, 1 + n_components + dp), dtype=np.float)
         Yi[:, 0] = 1
 
         M = np.zeros((N, N), dtype=np.float)
@@ -319,16 +337,17 @@ def locally_linear_embedding(
                 Ci = np.dot(Gi, Gi.T)
                 U = eigh(Ci)[1][:, ::-1]
 
-            Yi[:, 1:1 + out_dim] = U[:, :out_dim]
+            Yi[:, 1:1 + n_components] = U[:, :n_components]
 
-            j = 1 + out_dim
-            for k in range(out_dim):
-                Yi[:, j:j + out_dim - k] = U[:, k:k + 1] * U[:, k:out_dim]
-                j += out_dim - k
+            j = 1 + n_components
+            for k in range(n_components):
+                Yi[:, j:j + n_components - k] = \
+                        U[:, k:k + 1] * U[:, k:n_components]
+                j += n_components - k
 
             Q, R = qr(Yi)
 
-            w = Q[:, out_dim + 1:]
+            w = Q[:, n_components + 1:]
             S = w.sum(0)
 
             S[np.where(abs(S) < hessian_tol)] = 1
@@ -341,8 +360,9 @@ def locally_linear_embedding(
             M = csr_matrix(M)
 
     elif method == 'modified':
-        if n_neighbors < out_dim:
-            raise ValueError("modified LLE requires n_neighbors >= out_dim")
+        if n_neighbors < n_components:
+            raise ValueError("modified LLE requires "
+                "n_neighbors >= n_components")
 
         neighbors = nbrs.kneighbors(X, n_neighbors=n_neighbors + 1,
                                     return_distance=False)
@@ -388,7 +408,7 @@ def locally_linear_embedding(
 
         #calculate eta: the median of the ratio of small to large eigenvalues
         # across the points.  This is used to determine s_i, below
-        rho = evals[:, out_dim:].sum(1) / evals[:, :out_dim].sum(1)
+        rho = evals[:, n_components:].sum(1) / evals[:, :n_components].sum(1)
         eta = np.median(rho)
 
         #find s_i, the size of the "almost null space" for each point:
@@ -459,15 +479,15 @@ def locally_linear_embedding(
             Xi = X[neighbors[i]]
             Xi -= Xi.mean(0)
 
-            # compute out_dim largest eigenvalues of Xi * Xi^T
+            # compute n_components largest eigenvalues of Xi * Xi^T
             if use_svd:
                 v = svd(Xi, full_matrices=True)[0]
             else:
                 Ci = np.dot(Xi, Xi.T)
                 v = eigh(Ci)[1][:, ::-1]
 
-            Gi = np.zeros((n_neighbors, out_dim + 1))
-            Gi[:, 1:] = v[:, :out_dim]
+            Gi = np.zeros((n_neighbors, n_components + 1))
+            Gi[:, 1:] = v[:, :n_components]
             Gi[:, 0] = 1. / np.sqrt(n_neighbors)
 
             GiGiT = np.dot(Gi, Gi.T)
@@ -476,8 +496,8 @@ def locally_linear_embedding(
             M[nbrs_x, nbrs_y] -= GiGiT
             M[neighbors[i], neighbors[i]] += 1
 
-    return null_space(M, out_dim, k_skip=1, eigen_solver=eigen_solver,
-                      tol=tol, max_iter=max_iter)
+    return null_space(M, n_components, k_skip=1, eigen_solver=eigen_solver,
+                      tol=tol, max_iter=max_iter, random_state=random_state)
 
 
 class LocallyLinearEmbedding(BaseEstimator):
@@ -488,7 +508,7 @@ class LocallyLinearEmbedding(BaseEstimator):
     n_neighbors : integer
         number of neighbors to consider for each point.
 
-    out_dim : integer
+    n_components : integer
         number of coordinates for the manifold
 
     reg : float
@@ -519,7 +539,7 @@ class LocallyLinearEmbedding(BaseEstimator):
         standard : use the standard locally linear embedding algorithm.
                    see reference [1]
         hessian  : use the Hessian eigenmap method.  This method requires
-                   n_neighbors > out_dim * (1 + (out_dim + 1) / 2.
+                   n_neighbors > n_components * (1 + (n_components + 1) / 2.
                    see reference [2]
         modified : use the modified locally linear embedding algorithm.
                    see reference [3]
@@ -538,9 +558,13 @@ class LocallyLinearEmbedding(BaseEstimator):
         algorithm to use for nearest neighbors search,
         passed to neighbors.NearestNeighbors instance
 
+    random_state: numpy.RandomState, optional
+        The generator used to initialize the centers. Defaults to numpy.random.
+        Used to determine the starting vector for arpack iterations
+
     Attributes
     ----------
-    `embedding_vectors_` : array-like, shape [out_dim, n_samples]
+    `embedding_vectors_` : array-like, shape [n_components, n_samples]
         Stores the embedding vectors
 
     `reconstruction_error_` : float
@@ -549,14 +573,36 @@ class LocallyLinearEmbedding(BaseEstimator):
     `nbrs_` : NearestNeighbors object
         Stores nearest neighbors instance, including BallTree or KDtree
         if applicable.
+
+    References
+    ----------
+
+    .. [1] `Roweis, S. & Saul, L. Nonlinear dimensionality reduction
+        by locally linear embedding.  Science 290:2323 (2000).`
+    .. [2] `Donoho, D. & Grimes, C. Hessian eigenmaps: Locally
+        linear embedding techniques for high-dimensional data.
+        Proc Natl Acad Sci U S A.  100:5591 (2003).`
+    .. [3] `Zhang, Z. & Wang, J. MLLE: Modified Locally Linear
+        Embedding Using Multiple Weights.`
+        http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.70.382
+    .. [4] `Zhang, Z. & Zha, H. Principal manifolds and nonlinear
+        dimensionality reduction via tangent space alignment.
+        Journal of Shanghai Univ.  8:406 (2004)`
     """
 
-    def __init__(self, n_neighbors=5, out_dim=2, reg=1E-3,
-                 eigen_solver='auto', tol=1E-6, max_iter=100,
-                 method='standard', hessian_tol=1E-4, modified_tol=1E-12,
-                 neighbors_algorithm='auto'):
-        self.n_neighbors = n_neighbors
+    def __init__(self, n_neighbors=5, n_components=2, reg=1E-3,
+            eigen_solver='auto', tol=1E-6, max_iter=100, method='standard',
+            hessian_tol=1E-4, modified_tol=1E-12, neighbors_algorithm='auto',
+            random_state=None, out_dim=None):
+
+        if out_dim:
+            warnings.warn("Parameter ``out_dim`` was renamed to "
+                "``n_components`` and is now deprecated.", DeprecationWarning,
+                stacklevel=2)
         self.out_dim = out_dim
+
+        self.n_neighbors = n_neighbors
+        self.n_components = n_components
         self.reg = reg
         self.eigen_solver = eigen_solver
         self.tol = tol
@@ -564,17 +610,27 @@ class LocallyLinearEmbedding(BaseEstimator):
         self.method = method
         self.hessian_tol = hessian_tol
         self.modified_tol = modified_tol
+        self.random_state = random_state
         self.nbrs_ = NearestNeighbors(n_neighbors,
                                       algorithm=neighbors_algorithm)
 
     def _fit_transform(self, X):
+        if self.out_dim:
+            warnings.warn("Parameter ``out_dim`` was renamed to "
+                "``n_components`` and is now deprecated.", DeprecationWarning,
+                stacklevel=3)
+            self.n_components = self.out_dim
+            self.out_dim = None
+
+        self.random_state = check_random_state(self.random_state)
         self.nbrs_.fit(X)
         self.embedding_, self.reconstruction_error_ = \
             locally_linear_embedding(
-                self.nbrs_, self.n_neighbors, self.out_dim,
+                self.nbrs_, self.n_neighbors, self.n_components,
                 eigen_solver=self.eigen_solver, tol=self.tol,
                 max_iter=self.max_iter, method=self.method,
-                hessian_tol=self.hessian_tol, modified_tol=self.modified_tol)
+                hessian_tol=self.hessian_tol, modified_tol=self.modified_tol,
+                random_state=self.random_state)
 
     def fit(self, X, y=None):
         """Compute the embedding vectors for data X
@@ -601,7 +657,7 @@ class LocallyLinearEmbedding(BaseEstimator):
 
         Returns
         -------
-        X_new: array-like, shape (n_samples, out_dim)
+        X_new: array-like, shape (n_samples, n_components)
         """
         self._fit_transform(X)
         return self.embedding_
@@ -616,7 +672,7 @@ class LocallyLinearEmbedding(BaseEstimator):
 
         Returns
         -------
-        X_new : array, shape = [n_samples, out_dim]
+        X_new : array, shape = [n_samples, n_components]
 
         Notes
         -----
@@ -628,7 +684,7 @@ class LocallyLinearEmbedding(BaseEstimator):
                                     return_distance=False)
         weights = barycenter_weights(X, self.nbrs_._fit_X[ind],
                                      reg=self.reg)
-        X_new = np.empty((X.shape[0], self.out_dim))
+        X_new = np.empty((X.shape[0], self.n_components))
         for i in range(X.shape[0]):
             X_new[i] = np.dot(self.embedding_[ind[i]].T, weights[i])
         return X_new
