@@ -1,31 +1,27 @@
 from itertools import product
 import numpy as np
 from numpy.testing import assert_almost_equal, assert_array_almost_equal
+from nose.tools import assert_true
+
 
 from sklearn import datasets
 from sklearn import manifold
 from sklearn import neighbors
 from sklearn import pipeline
 from sklearn import preprocessing
+from sklearn.utils.testing import assert_less
 
 eigen_solvers = ['auto', 'dense', 'arpack']
 path_methods = ['auto', 'FW', 'D']
 
 
-def assert_lower(a, b, details=None):
-    message = "%r is not lower than %r" % (a, b)
-    if details is not None:
-        message += ": " + details
-    assert a < b, message
-
-
 def test_isomap_simple_grid():
     # Isomap should preserve distances when all neighbors are used
     N_per_side = 5
     Npts = N_per_side ** 2
     n_neighbors = Npts - 1
 
-    # grid of equidistant points in 2D, out_dim = n_dim
+    # grid of equidistant points in 2D, n_components = n_dim
     X = np.array(list(product(range(N_per_side), repeat=2)))
 
     # distances from each point to all others
@@ -34,7 +30,7 @@ def test_isomap_simple_grid():
 
     for eigen_solver in eigen_solvers:
         for path_method in path_methods:
-            clf = manifold.Isomap(n_neighbors=n_neighbors, out_dim=2,
+            clf = manifold.Isomap(n_neighbors=n_neighbors, n_components=2,
                                   eigen_solver=eigen_solver,
                                   path_method=path_method)
             clf.fit(X)
@@ -51,7 +47,7 @@ def test_isomap_reconstruction_error():
     Npts = N_per_side ** 2
     n_neighbors = Npts - 1
 
-    # grid of equidistant points in 2D, out_dim = n_dim
+    # grid of equidistant points in 2D, n_components = n_dim
     X = np.array(list(product(range(N_per_side), repeat=2)))
 
     # add noise in a third dimension
@@ -68,7 +64,7 @@ def test_isomap_reconstruction_error():
 
     for eigen_solver in eigen_solvers:
         for path_method in path_methods:
-            clf = manifold.Isomap(n_neighbors=n_neighbors, out_dim=2,
+            clf = manifold.Isomap(n_neighbors=n_neighbors, n_components=2,
                                   eigen_solver=eigen_solver,
                                   path_method=path_method)
             clf.fit(X)
@@ -104,17 +100,19 @@ def test_transform():
     X_iso2 = iso.transform(X + noise)
 
     # Make sure the rms error on re-embedding is comparable to noise_scale
-    assert np.sqrt(np.mean((X_iso - X_iso2) ** 2)) < 2 * noise_scale
+    assert_less(np.sqrt(np.mean((X_iso - X_iso2) ** 2)), 2 * noise_scale)
 
 
 def test_pipeline():
     # check that Isomap works fine as a transformer in a Pipeline
-    iris = datasets.load_iris()
+    # only checks that no error is raised.
+    # TODO check that it actually does something useful
+    X, y = datasets.make_blobs(random_state=0)
     clf = pipeline.Pipeline(
         [('isomap', manifold.Isomap()),
-         ('neighbors_clf', neighbors.KNeighborsClassifier())])
-    clf.fit(iris.data, iris.target)
-    assert_lower(.7, clf.score(iris.data, iris.target))
+         ('clf', neighbors.KNeighborsClassifier())])
+    clf.fit(X, y)
+    assert_less(.9, clf.score(X, y))
 
 
 if __name__ == '__main__':
