@@ -7,7 +7,8 @@ from .metrics import confusion_matrix, roc_curve, auc, precision_score, \
                 recall_score, fbeta_score, f1_score, zero_one_score, \
                 precision_recall_fscore_support, classification_report, \
                 precision_recall_curve, explained_variance_score, r2_score, \
-                zero_one, mean_square_error, hinge_loss
+                zero_one, mean_square_error, hinge_loss, matthews_corrcoef, \
+                mean_squared_error
 
 from . import cluster
 from .cluster import adjusted_rand_score
