@@ -26,23 +26,24 @@ class DPGMMTester(GMMTester):
     do_test_eval = False
 
     def score(self, g, train_obs):
-        return g.lower_bound()
+        _, z = g.eval(train_obs)
+        return g.lower_bound(train_obs, z)
 
 
 class TestDPGMMWithSphericalCovars(unittest.TestCase, DPGMMTester):
-    cvtype = 'spherical'
+    covariance_type = 'spherical'
 
 
 class TestDPGMMWithDiagCovars(unittest.TestCase, DPGMMTester):
-    cvtype = 'diag'
+    covariance_type = 'diag'
 
 
 class TestDPGMMWithTiedCovars(unittest.TestCase, DPGMMTester):
-    cvtype = 'tied'
+    covariance_type = 'tied'
 
 
 class TestDPGMMWithFullCovars(unittest.TestCase, DPGMMTester):
-    cvtype = 'full'
+    covariance_type = 'full'
 
 
 class VBGMMTester(GMMTester):
@@ -50,23 +51,24 @@ class VBGMMTester(GMMTester):
     do_test_eval = False
 
     def score(self, g, train_obs):
-        return g.lower_bound()
+        _, z = g.eval(train_obs)
+        return g.lower_bound(train_obs, z)
 
 
 class TestVBGMMWithSphericalCovars(unittest.TestCase, VBGMMTester):
-    cvtype = 'spherical'
+    covariance_type = 'spherical'
 
 
 class TestVBGMMWithDiagCovars(unittest.TestCase, VBGMMTester):
-    cvtype = 'diag'
+    covariance_type = 'diag'
 
 
 class TestVBGMMWithTiedCovars(unittest.TestCase, VBGMMTester):
-    cvtype = 'tied'
+    covariance_type = 'tied'
 
 
 class TestVBGMMWithFullCovars(unittest.TestCase, VBGMMTester):
-    cvtype = 'full'
+    covariance_type = 'full'
 
 
 if __name__ == '__main__':
