@@ -57,7 +57,7 @@ class BaseNB(BaseEstimator, ClassifierMixin):
             Predicted target values for X
         """
         jll = self._joint_log_likelihood(X)
-        return self._classes[np.argmax(jll, axis=1)]
+        return self.classes_[np.argmax(jll, axis=1)]
 
     def predict_log_proba(self, X):
         """
@@ -153,28 +153,34 @@ class GaussianNB(BaseNB):
         X = np.asarray(X)
         y = np.asarray(y)
 
-        self._classes = unique_y = np.unique(y)
-        n_classes = unique_y.shape[0]
-        _, n_features = X.shape
+        n_samples, n_features = X.shape
 
-        self.theta_ = np.empty((n_classes, n_features))
-        self.sigma_ = np.empty((n_classes, n_features))
-        self.class_prior_ = np.empty(n_classes)
+        if n_samples != y.shape[0]:
+            raise ValueError("X and y have incompatible shapes")
+
+        self.classes_ = unique_y = np.unique(y)
+        n_classes = unique_y.shape[0]
+
+        self.theta_ = np.zeros((n_classes, n_features))
+        self.sigma_ = np.zeros((n_classes, n_features))
+        self.class_prior_ = np.zeros(n_classes)
+        epsilon = 1e-9
         for i, y_i in enumerate(unique_y):
             self.theta_[i, :] = np.mean(X[y == y_i, :], axis=0)
-            self.sigma_[i, :] = np.var(X[y == y_i, :], axis=0)
-            self.class_prior_[i] = np.float(np.sum(y == y_i)) / n_classes
+            self.sigma_[i, :] = np.var(X[y == y_i, :], axis=0) + epsilon
+            self.class_prior_[i] = np.float(np.sum(y == y_i)) / n_samples
         return self
 
     def _joint_log_likelihood(self, X):
         X = array2d(X)
         joint_log_likelihood = []
-        for i in xrange(np.size(self._classes)):
+        for i in xrange(np.size(self.classes_)):
             jointi = np.log(self.class_prior_[i])
             n_ij = - 0.5 * np.sum(np.log(np.pi * self.sigma_[i, :]))
             n_ij -= 0.5 * np.sum(((X - self.theta_[i, :]) ** 2) / \
-                                    (self.sigma_[i, :]), 1)
+                                 (self.sigma_[i, :]), 1)
             joint_log_likelihood.append(jointi + n_ij)
+
         joint_log_likelihood = np.array(joint_log_likelihood).T
         return joint_log_likelihood
 
@@ -237,8 +243,8 @@ class BaseDiscreteNB(BaseNB):
 
         labelbin = LabelBinarizer()
         Y = labelbin.fit_transform(y)
-        self._classes = labelbin.classes_
-        n_classes = len(self._classes)
+        self.classes_ = labelbin.classes_
+        n_classes = len(self.classes_)
         if Y.shape[1] == 1:
             Y = np.concatenate((1 - Y, Y), axis=1)
 
@@ -253,8 +259,9 @@ class BaseDiscreteNB(BaseNB):
             Y *= array2d(sample_weight).T
 
         if class_prior:
-            assert len(class_prior) == n_classes, \
-                   'Number of priors must match number of classes'
+            if len(class_prior) != n_classes:
+                raise ValueError(
+                        "Number of priors must match number of classes")
             self.class_log_prior_ = np.log(class_prior)
         elif self.fit_prior:
             # empirical prior, with sample_weight taken into account
@@ -284,8 +291,18 @@ class BaseDiscreteNB(BaseNB):
 
         return N_c, N_c_i
 
-    intercept_ = property(lambda self: self.class_log_prior_)
-    coef_ = property(lambda self: self.feature_log_prob_)
+    # XXX The following is a stopgap measure; we need to set the dimensions
+    # of class_log_prior_ and feature_log_prob_ correctly.
+    def _get_coef(self):
+        return self.feature_log_prob_[1] if len(self.classes_) == 2 \
+                                         else self.feature_log_prob_
+
+    def _get_intercept(self):
+        return self.class_log_prior_[1] if len(self.classes_) == 2 \
+                                        else self.class_log_prior_
+
+    coef_ = property(_get_coef)
+    intercept_ = property(_get_intercept)
 
 
 class MultinomialNB(BaseDiscreteNB):
@@ -345,7 +362,8 @@ class MultinomialNB(BaseDiscreteNB):
     def _joint_log_likelihood(self, X):
         """Calculate the posterior log probability of the samples X"""
         X = atleast2d_or_csr(X)
-        return safe_sparse_dot(X, self.coef_.T) + self.intercept_
+        return (safe_sparse_dot(X, self.feature_log_prob_.T)
+               + self.class_log_prior_)
 
 
 class BernoulliNB(BaseDiscreteNB):
@@ -355,8 +373,6 @@ class BernoulliNB(BaseDiscreteNB):
     difference is that while MultinomialNB works with occurrence counts,
     BernoulliNB is designed for binary/boolean features.
 
-    Note: this class does not check whether features are actually boolean.
-
     Parameters
     ----------
     alpha: float, optional (default=1.0)
@@ -389,9 +405,8 @@ class BernoulliNB(BaseDiscreteNB):
     >>> print clf.predict(X[2])
     [3]
 
-    Notes
-    -----
-    **References**:
+    References
+    ----------
 
     C.D. Manning, P. Raghavan and H. Schütze (2008). Introduction to
     Information Retrieval. Cambridge University Press, pp. 234–265.
@@ -433,6 +448,6 @@ class BernoulliNB(BaseDiscreteNB):
         # Compute  neg_prob · (1 - X).T  as  ∑neg_prob - X · neg_prob
         X_neg_prob = (neg_prob.sum(axis=1)
                     - safe_sparse_dot(X, neg_prob.T))
-        jll = safe_sparse_dot(X, self.coef_.T) + X_neg_prob
+        jll = safe_sparse_dot(X, self.feature_log_prob_.T) + X_neg_prob
 
-        return jll + self.intercept_
+        return jll + self.class_log_prior_
