@@ -16,7 +16,7 @@ from .base import \
     NeighborsBase, KNeighborsMixin,\
     RadiusNeighborsMixin, SupervisedIntegerMixin
 from ..base import ClassifierMixin
-from ..utils import atleast2d_or_csr, deprecated
+from ..utils import atleast2d_or_csr
 
 
 class KNeighborsClassifier(NeighborsBase, KNeighborsMixin,
@@ -68,6 +68,12 @@ class KNeighborsClassifier(NeighborsBase, KNeighborsMixin,
         ordering of the training data.
         If the fit method is ``'kd_tree'``, no warnings will be generated.
 
+    p: integer, optional (default = 2)
+        Parameter for the Minkowski metric from
+        sklearn.metrics.pairwise.pairwise_distances. When p = 1, this is
+        equivalent to using manhattan_distance (l1), and euclidean_distance
+        (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
+
     Examples
     --------
     >>> X = [[0], [1], [2], [3]]
@@ -97,11 +103,12 @@ class KNeighborsClassifier(NeighborsBase, KNeighborsMixin,
     def __init__(self, n_neighbors=5,
                  weights='uniform',
                  algorithm='auto', leaf_size=30,
-                 warn_on_equidistant=True):
+                 warn_on_equidistant=True, p=2):
         self._init_params(n_neighbors=n_neighbors,
                           algorithm=algorithm,
                           leaf_size=leaf_size,
-                          warn_on_equidistant=warn_on_equidistant)
+                          warn_on_equidistant=warn_on_equidistant,
+                          p=p)
         self.weights = _check_weights(weights)
 
     def predict(self, X):
@@ -174,6 +181,17 @@ class RadiusNeighborsClassifier(NeighborsBase, RadiusNeighborsMixin,
         required to store the tree.  The optimal value depends on the
         nature of the problem.
 
+    p: integer, optional (default = 2)
+        Parameter for the Minkowski metric from
+        sklearn.metrics.pairwise.pairwise_distances. When p = 1, this is
+        equivalent to using manhattan_distance (l1), and euclidean_distance
+        (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
+
+    outlier_label: int, optional (default = None)
+        Label, which is given for outlier samples (samples with no
+        neighbors on given radius).
+        If set to None, ValueError is raised, when outlier is detected.
+
     Examples
     --------
     >>> X = [[0], [1], [2], [3]]
@@ -201,11 +219,13 @@ class RadiusNeighborsClassifier(NeighborsBase, RadiusNeighborsMixin,
     """
 
     def __init__(self, radius=1.0, weights='uniform',
-                 algorithm='auto', leaf_size=30):
+                 algorithm='auto', leaf_size=30, p=2, outlier_label=None):
         self._init_params(radius=radius,
                           algorithm=algorithm,
-                          leaf_size=leaf_size)
+                          leaf_size=leaf_size,
+                          p=p)
         self.weights = _check_weights(weights)
+        self.outlier_label = outlier_label
 
     def predict(self, X):
         """Predict the class labels for the provided data
@@ -225,6 +245,24 @@ class RadiusNeighborsClassifier(NeighborsBase, RadiusNeighborsMixin,
         neigh_dist, neigh_ind = self.radius_neighbors(X)
         pred_labels = [self._y[ind] for ind in neigh_ind]
 
+        if self.outlier_label:
+            outlier_label = np.array((self.outlier_label, ))
+            small_value = np.array((1e-6, ))
+            for i, pl in enumerate(pred_labels):
+                # Check that all have at least 1 neighbor
+                if len(pl) < 1:
+                    pred_labels[i] = outlier_label
+                    neigh_dist[i] = small_value
+        else:
+            for pl in pred_labels:
+                # Check that all have at least 1 neighbor
+                if len(pl) < 1:
+                    raise ValueError('no neighbors found for a test sample, '
+                                     'you can try using larger radius, '
+                                     'give a label for outliers, '
+                                     'or consider removing them in your '
+                                     'dataset')
+
         weights = _get_weights(neigh_dist, self.weights)
 
         if weights is None:
@@ -236,112 +274,3 @@ class RadiusNeighborsClassifier(NeighborsBase, RadiusNeighborsMixin,
                               dtype=np.int)
 
         return mode.flatten().astype(np.int)
-
-
-@deprecated("""to be removed in v0.11;
-use KNeighborsClassifier or RadiusNeighborsClassifier instead""")
-class NeighborsClassifier(NeighborsBase, KNeighborsMixin,
-                          RadiusNeighborsMixin, SupervisedIntegerMixin,
-                          ClassifierMixin):
-    """Classifier implementing the nearest neighbors vote. (Deprecated)
-
-    DEPRECATED IN VERSION 0.9; WILL BE REMOVED IN VERSION 0.11
-    Please use :class:`KNeighborsClassifier` or
-    :class:`RadiusNeighborsClassifier` instead.
-
-    Samples participating in the vote are either the k-nearest neighbors
-    (for some k)  or all neighbors within some fixed radius around the sample
-    to classify.
-
-    Parameters
-    ----------
-    n_neighbors : int, optional (default = 5)
-        Number of neighbors to use by default for :meth:`k_neighbors` queries.
-
-    radius : float, optional (default = 1.0)
-        Range of parameter space to use by default for :meth`radius_neighbors`
-        queries.
-
-    algorithm : {'auto', 'ball_tree', 'kd_tree', 'brute'}, optional
-        Algorithm used to compute the nearest neighbors:
-
-        - 'ball_tree' will use :class:`BallTree`
-        - 'kd_tree' will use :class:`scipy.spatial.cKDtree`
-        - 'brute' will use a brute-force search.
-        - 'auto' will attempt to decide the most appropriate algorithm
-          based on the values passed to :meth:`fit` method.
-
-        Note: fitting on sparse input will override the setting of
-        this parameter, using brute force.
-
-    leaf_size : int, optional (default = 30)
-        Leaf size passed to BallTree or cKDTree.  This can affect the
-        speed of the construction and query, as well as the memory
-        required to store the tree.  The optimal value depends on the
-        nature of the problem.
-
-    classification_type : {'knn_vote', 'radius_vote'}, optional
-        Type of fit to use: 'knn_vote' specifies a k-NN classification.
-        'radius_vote' specifies a r-NN classification.  Default is 'knn_vote'.
-
-    Examples
-    --------
-    >>> X = [[0], [1], [2], [3]]
-    >>> y = [0, 0, 1, 1]
-    >>> from sklearn.neighbors import NeighborsClassifier
-    >>> neigh = NeighborsClassifier(n_neighbors=2)
-    >>> neigh.fit(X, y)
-    NeighborsClassifier(algorithm='auto', classification_type='knn_vote',
-              leaf_size=30, n_neighbors=2, radius=1.0)
-    >>> print neigh.predict([[1.5]])
-    [0]
-
-    See also
-    --------
-    NearestNeighbors
-    NeighborsRegressor
-
-    Notes
-    -----
-    See :ref:`Nearest Neighbors <neighbors>` in the online documentation
-    for a discussion of the choice of ``algorithm`` and ``leaf_size``.
-
-    http://en.wikipedia.org/wiki/K-nearest_neighbor_algorithm
-    """
-
-    def __init__(self, n_neighbors=5, radius=1.0,
-                 algorithm='auto', leaf_size=30,
-                 classification_type='knn_vote'):
-        if classification_type not in ('radius_vote', 'knn_vote'):
-            raise ValueError("classification_type not recognized")
-        self.classification_type = classification_type
-        self._init_params(n_neighbors=n_neighbors,
-                          radius=radius,
-                          algorithm=algorithm,
-                          leaf_size=leaf_size)
-
-    def predict(self, X):
-        """Predict the class labels for the provided data
-
-        Parameters
-        ----------
-        X: array
-            A 2-D array representing the test points.
-
-        Returns
-        -------
-        labels: array
-            List of class labels (one for each data sample).
-        """
-        X = atleast2d_or_csr(X)
-
-        if self.classification_type == 'knn_vote':
-            neigh_ind = self.kneighbors(X, return_distance=False)
-            pred_labels = self._y[neigh_ind]
-            mode, _ = stats.mode(pred_labels, axis=1)
-            return mode.flatten().astype(np.int)
-        else:
-            neigh_ind = self.radius_neighbors(X, return_distance=False)
-            pred_labels = [self._y[ind] for ind in neigh_ind]
-            return np.asarray([stats.mode(pi) for pi in pred_labels],
-                              dtype=np.int)
