@@ -14,7 +14,7 @@ from .base import \
     NeighborsBase, KNeighborsMixin, \
     RadiusNeighborsMixin, SupervisedFloatMixin
 from ..base import RegressorMixin
-from ..utils import atleast2d_or_csr, deprecated
+from ..utils import atleast2d_or_csr
 
 
 class KNeighborsRegressor(NeighborsBase, KNeighborsMixin,
@@ -70,6 +70,12 @@ class KNeighborsRegressor(NeighborsBase, KNeighborsMixin,
         ordering of the training data.
         If the fit method is ``'kd_tree'``, no warnings will be generated.
 
+    p: integer, optional (default = 2)
+        Parameter for the Minkowski metric from
+        sklearn.metrics.pairwise.pairwise_distances. When p = 1, this is
+        equivalent to using manhattan_distance (l1), and euclidean_distance
+        (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
+
     Examples
     --------
     >>> X = [[0], [1], [2], [3]]
@@ -97,11 +103,13 @@ class KNeighborsRegressor(NeighborsBase, KNeighborsMixin,
     """
 
     def __init__(self, n_neighbors=5, weights='uniform',
-                 algorithm='auto', leaf_size=30, warn_on_equidistant=True):
+                 algorithm='auto', leaf_size=30, warn_on_equidistant=True,
+                 p=2):
         self._init_params(n_neighbors=n_neighbors,
                           algorithm=algorithm,
                           leaf_size=leaf_size,
-                          warn_on_equidistant=warn_on_equidistant)
+                          warn_on_equidistant=warn_on_equidistant,
+                          p=p)
         self.weights = _check_weights(weights)
 
     def predict(self, X):
@@ -177,6 +185,13 @@ class RadiusNeighborsRegressor(NeighborsBase, RadiusNeighborsMixin,
         required to store the tree.  The optimal value depends on the
         nature of the problem.
 
+    p: integer, optional (default = 2)
+        Parameter for the Minkowski metric from
+        sklearn.metrics.pairwise.pairwise_distances. When p = 1, this is
+        equivalent to using manhattan_distance (l1), and euclidean_distance
+        (l2) for p = 2. For arbitrary p, minkowski_distance (l_p) is used.
+
+
     Examples
     --------
     >>> X = [[0], [1], [2], [3]]
@@ -204,10 +219,11 @@ class RadiusNeighborsRegressor(NeighborsBase, RadiusNeighborsMixin,
     """
 
     def __init__(self, radius=1.0, weights='uniform',
-                 algorithm='auto', leaf_size=30):
+                 algorithm='auto', leaf_size=30, p=2):
         self._init_params(radius=radius,
                           algorithm=algorithm,
-                          leaf_size=leaf_size)
+                          leaf_size=leaf_size,
+                          p=p)
         self.weights = _check_weights(weights)
 
     def predict(self, X):
@@ -236,118 +252,3 @@ class RadiusNeighborsRegressor(NeighborsBase, RadiusNeighborsMixin,
             return np.array([(np.sum(self._y[ind] * weights[i])
                               / np.sum(weights[i]))
                              for (i, ind) in enumerate(neigh_ind)])
-
-
-@deprecated("""will be removed in v0.11;
-use KNeighborsRegressor or RadiusNeighborsRegressor instead""")
-class NeighborsRegressor(NeighborsBase, KNeighborsMixin, RadiusNeighborsMixin,
-                         SupervisedFloatMixin,
-                         RegressorMixin):
-    """Regression based on nearest neighbors. (Deprecated)
-
-    DEPRECATED IN VERSION 0.9; WILL BE REMOVED IN VERSION 0.11
-    Please use :class:`KNeighborsRegressor` or
-    :class:`RadiusNeighborsRegressor` instead.
-
-    The target is predicted by local interpolation of the targets
-    associated of the nearest neighbors in the training set.
-    Samples used for the regression are either the k-nearest points, or
-    all points within some fixed radius.
-
-    Parameters
-    ----------
-    n_neighbors : int, optional (default = 5)
-        Number of neighbors to use by default for :meth:`k_neighbors` queries.
-
-    radius : float, optional (default = 1.0)
-        Range of parameter space to use by default for :meth`radius_neighbors`
-        queries.
-
-    algorithm : {'auto', 'ball_tree', 'kd_tree', 'brute'}, optional
-        Algorithm used to compute the nearest neighbors:
-
-        - 'ball_tree' will use :class:`BallTree`
-        - 'kd_tree' will use :class:`scipy.spatial.cKDtree`
-        - 'brute' will use a brute-force search.
-        - 'auto' will attempt to decide the most appropriate algorithm
-          based on the values passed to :meth:`fit` method.
-
-        Note: fitting on sparse input will override the setting of
-        this parameter, using brute force.
-
-    leaf_size : int, optional (default = 30)
-        Leaf size passed to BallTree or cKDTree.  This can affect the
-        speed of the construction and query, as well as the memory
-        required to store the tree.  The optimal value depends on the
-        nature of the problem.
-
-    classification_type : {'knn_vote', 'radius_vote'}, optional
-        Type of fit to use: 'knn_vote' specifies a k-NN classification.
-        'radius_vote' specifies a r-NN classification.  Default is 'knn_vote'.
-
-    Examples
-    --------
-    >>> X = [[0], [1], [2], [3]]
-    >>> y = [0, 0, 1, 1]
-    >>> from sklearn.neighbors import NeighborsRegressor
-    >>> neigh = NeighborsRegressor(n_neighbors=2)
-    >>> neigh.fit(X, y)
-    NeighborsRegressor(algorithm='auto', classification_type='knn_vote',
-              leaf_size=30, n_neighbors=2, radius=1.0)
-    >>> print neigh.predict([[1.5]])
-    [ 0.5]
-
-    See also
-    --------
-    NearestNeighbors
-    KNeighborsRegressor
-    RadiusNeighborsRegressor
-    KNeighborsClassifier
-    RadiusNeighborsClassifier
-
-    Notes
-    -----
-    See :ref:`Nearest Neighbors <neighbors>` in the online documentation
-    for a discussion of the choice of ``algorithm`` and ``leaf_size``.
-
-    http://en.wikipedia.org/wiki/K-nearest_neighbor_algorithm
-    """
-
-    def __init__(self, n_neighbors=5, radius=1.0,
-                 algorithm='auto',
-                 leaf_size=30, classification_type='knn_vote'):
-        if classification_type not in ('radius_vote', 'knn_vote'):
-            raise ValueError("classification_type not recognized")
-        self.classification_type = classification_type
-        self._init_params(n_neighbors=n_neighbors,
-                          radius=radius,
-                          algorithm=algorithm,
-                          leaf_size=leaf_size)
-
-    def predict(self, X):
-        """Predict the target for the provided data
-
-        Parameters
-        ----------
-        X : array
-            A 2-D array representing the test data.
-
-        Returns
-        -------
-        y: array
-            List of target values (one for each data sample).
-        """
-        X = atleast2d_or_csr(X)
-
-        if self.classification_type == 'knn_vote':
-            neigh_ind = self.kneighbors(X, return_distance=False)
-
-            # compute interpolation on y
-            return np.mean(self._y[neigh_ind], axis=1)
-
-        else:
-            neigh_ind = self.radius_neighbors(X, return_distance=False)
-
-            # compute interpolation on y
-            return np.array([np.mean(self._y[ind])
-                             for ind in neigh_ind])
