@@ -1,11 +1,17 @@
+import os
+
 def configuration(parent_package='', top_path=None):
     import numpy
     from numpy.distutils.misc_util import Configuration
 
     config = Configuration('neighbors', parent_package, top_path)
+    libraries = []
+    if os.name == 'posix':
+        libraries.append('m')
 
     config.add_extension('ball_tree',
                          sources=['ball_tree.c'],
-                         include_dirs=[numpy.get_include()])
+                         include_dirs=[numpy.get_include()],
+                         libraries=libraries)
 
     return config
