@@ -1,3 +1,4 @@
+import os
 from os.path import join
 import warnings
 
@@ -5,6 +6,11 @@ import warnings
 def configuration(parent_package='', top_path=None):
     from numpy.distutils.misc_util import Configuration
     from numpy.distutils.system_info import get_info, BlasNotFoundError
+    import numpy
+
+    libraries = []
+    if os.name == 'posix':
+        libraries.append('m')
 
     config = Configuration('sklearn', parent_package, top_path)
 
@@ -24,7 +30,6 @@ def configuration(parent_package='', top_path=None):
     config.add_subpackage("ensemble/tests")
     config.add_subpackage('feature_selection')
     config.add_subpackage('feature_selection/tests')
-    config.add_subpackage('preprocessing')
     config.add_subpackage('utils')
     config.add_subpackage('utils/tests')
     config.add_subpackage('externals')
@@ -35,6 +40,7 @@ def configuration(parent_package='', top_path=None):
     config.add_subpackage('neighbors')
     config.add_subpackage('manifold')
     config.add_subpackage('metrics')
+    config.add_subpackage('semi_supervised')
     config.add_subpackage("tree")
     config.add_subpackage("tree/tests")
     config.add_subpackage('metrics/tests')
@@ -42,6 +48,14 @@ def configuration(parent_package='', top_path=None):
     config.add_subpackage('metrics/cluster')
     config.add_subpackage('metrics/cluster/tests')
 
+    # add cython extension module for hmm
+    config.add_extension(
+        '_hmmc',
+        sources=['_hmmc.c'],
+        include_dirs=[numpy.get_include()],
+        libraries=libraries,
+    )
+
     # some libs needs cblas, fortran-compiled BLAS will not be sufficient
     blas_info = get_info('blas_opt', 0)
     if (not blas_info) or (
