@@ -1,7 +1,6 @@
 import numpy as np
-from scipy import linalg
 from scipy import sparse
-from sklearn import datasets, svm, linear_model
+from sklearn import datasets, svm, linear_model, base
 from numpy.testing import assert_array_almost_equal, \
      assert_array_equal, assert_equal
 
@@ -11,6 +10,7 @@ from sklearn.svm.tests import test_svm
 
 # test sample 1
 X = np.array([[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]])
+X_sp = sparse.lil_matrix(X)
 Y = [1, 1, 1, 2, 2, 2]
 T = np.array([[-1, -1], [2, 2], [3, 2]])
 true_result = [1, 2, 2]
@@ -18,6 +18,7 @@ true_result = [1, 2, 2]
 # test sample 2
 X2 = np.array([[0, 0, 0], [1, 1, 1], [2, 0, 0, ],
                [0, 0, 2], [3, 3, 3]])
+X2_sp = sparse.dok_matrix(X2)
 Y2 = [1, 2, 2, 2, 3]
 T2 = np.array([[-1, -1, -1], [1, 1, 1], [2, 2, 2]])
 true_result2 = [1, 2, 3]
@@ -25,35 +26,36 @@ true_result2 = [1, 2, 3]
 
 iris = datasets.load_iris()
 # permute
-perm = np.random.permutation(iris.target.size)
+rng = np.random.RandomState(0)
+perm = rng.permutation(iris.target.size)
 iris.data = iris.data[perm]
 iris.target = iris.target[perm]
 # sparsify
 iris.data = sparse.csr_matrix(iris.data)
 
 
-def test_SVC():
+def test_svc():
     """Check that sparse SVC gives the same result as SVC"""
 
     clf = svm.SVC(kernel='linear').fit(X, Y)
-    sp_clf = svm.sparse.SVC(kernel='linear').fit(X, Y)
+    sp_clf = svm.SVC(kernel='linear').fit(X_sp, Y)
 
     assert_array_equal(sp_clf.predict(T), true_result)
 
-    assert sparse.issparse(sp_clf.support_vectors_)
+    assert_true(sparse.issparse(sp_clf.support_vectors_))
     assert_array_almost_equal(clf.support_vectors_,
             sp_clf.support_vectors_.todense())
 
-    assert sparse.issparse(sp_clf.dual_coef_)
+    assert_true(sparse.issparse(sp_clf.dual_coef_))
     assert_array_almost_equal(clf.dual_coef_, sp_clf.dual_coef_.todense())
 
-    assert sparse.issparse(sp_clf.coef_)
+    assert_true(sparse.issparse(sp_clf.coef_))
     assert_array_almost_equal(clf.coef_, sp_clf.coef_.todense())
     assert_array_almost_equal(clf.predict(T), sp_clf.predict(T))
 
     # refit with a different dataset
     clf.fit(X2, Y2)
-    sp_clf.fit(X2, Y2)
+    sp_clf.fit(X2_sp, Y2)
     assert_array_almost_equal(clf.support_vectors_,
             sp_clf.support_vectors_.todense())
     assert_array_almost_equal(clf.dual_coef_, sp_clf.dual_coef_.todense())
@@ -61,10 +63,10 @@ def test_SVC():
     assert_array_almost_equal(clf.predict(T2), sp_clf.predict(T2))
 
 
-def test_SVC_iris():
+def test_svc_iris():
     """Test the sparse SVC with the iris dataset"""
     for k in ('linear', 'poly', 'rbf'):
-        sp_clf = svm.sparse.SVC(kernel=k).fit(iris.data, iris.target)
+        sp_clf = svm.SVC(kernel=k).fit(iris.data, iris.target)
         clf = svm.SVC(kernel=k).fit(iris.data.todense(), iris.target)
 
         assert_array_almost_equal(clf.support_vectors_,
@@ -84,40 +86,40 @@ def test_error():
     assert_raises(ValueError, svm.SVC(C=-1).fit, X, Y)
 
     # impossible value of nu
-    clf = svm.sparse.NuSVC(nu=0.0)
-    assert_raises(ValueError, clf.fit, X, Y)
+    clf = svm.NuSVC(nu=0.0)
+    assert_raises(ValueError, clf.fit, X_sp, Y)
 
     Y2 = Y[:-1]  # wrong dimensions for labels
-    assert_raises(ValueError, clf.fit, X, Y2)
+    assert_raises(ValueError, clf.fit, X_sp, Y2)
 
-    clf = svm.sparse.SVC()
-    clf.fit(X, Y)
+    clf = svm.SVC()
+    clf.fit(X_sp, Y)
     assert_array_equal(clf.predict(T), true_result)
 
 
-def test_LinearSVC():
+def test_linearsvc():
     """
     Similar to test_SVC
     """
     clf = svm.LinearSVC().fit(X, Y)
-    sp_clf = svm.sparse.LinearSVC().fit(X, Y)
+    sp_clf = svm.LinearSVC().fit(X_sp, Y)
 
-    assert sp_clf.fit_intercept
+    assert_true(sp_clf.fit_intercept)
 
     assert_array_almost_equal(clf.raw_coef_, sp_clf.raw_coef_, decimal=4)
 
-    assert_array_almost_equal(clf.predict(X), sp_clf.predict(X))
+    assert_array_almost_equal(clf.predict(X), sp_clf.predict(X_sp))
 
     clf.fit(X2, Y2)
-    sp_clf.fit(X2, Y2)
+    sp_clf.fit(X2_sp, Y2)
 
     assert_array_almost_equal(clf.raw_coef_, sp_clf.raw_coef_, decimal=4)
 
 
-def test_LinearSVC_iris():
+def test_linearsvc_iris():
     """Test the sparse LinearSVC with the iris dataset"""
 
-    sp_clf = svm.sparse.LinearSVC().fit(iris.data, iris.target)
+    sp_clf = svm.LinearSVC().fit(iris.data, iris.target)
     clf = svm.LinearSVC().fit(iris.data.todense(), iris.target)
 
     assert_array_almost_equal(clf.label_, sp_clf.label_)
@@ -136,29 +138,29 @@ def test_weight():
     """
     Test class weights
     """
-
     X_, y_ = make_classification(n_samples=200, n_features=100,
                                  weights=[0.833, 0.167], random_state=0)
 
     X_ = sparse.csr_matrix(X_)
-    for clf in (linear_model.sparse.LogisticRegression(),
-                svm.sparse.LinearSVC(),
-                svm.sparse.SVC()):
-        clf.fit(X_[:180], y_[:180], class_weight={0: 5})
+    for clf in (linear_model.LogisticRegression(),
+                svm.LinearSVC(),
+                svm.SVC()):
+        clf.set_params(class_weight={0: 5})
+        clf.fit(X_[:180], y_[:180])
         y_pred = clf.predict(X_[180:])
-        assert np.sum(y_pred == y_[180:]) >= 11
+        assert_true(np.sum(y_pred == y_[180:]) >= 11)
 
 
 def test_sample_weights():
     """
     Test weights on individual samples
     """
-    clf = svm.sparse.SVC()
-    clf.fit(X, Y)
+    clf = svm.SVC()
+    clf.fit(X_sp, Y)
     assert_array_equal(clf.predict(X[2]), [1.])
 
     sample_weight = [.1] * 3 + [10] * 3
-    clf.fit(X, Y, sample_weight=sample_weight)
+    clf.fit(X_sp, Y, sample_weight=sample_weight)
     assert_array_equal(clf.predict(X[2]), [2.])
 
 
@@ -166,7 +168,7 @@ def test_sparse_liblinear_intercept_handling():
     """
     Test that sparse liblinear honours intercept_scaling param
     """
-    test_svm.test_dense_liblinear_intercept_handling(svm.sparse.LinearSVC)
+    test_svm.test_dense_liblinear_intercept_handling(svm.LinearSVC)
 
 
 def test_sparse_realdata():
@@ -195,37 +197,20 @@ def test_sparse_realdata():
         1.,  3.])
 
     clf = svm.SVC(kernel='linear').fit(X.todense(), y)
-    sp_clf = svm.sparse.SVC(kernel='linear').fit(X, y)
+    sp_clf = svm.SVC(kernel='linear').fit(sparse.coo_matrix(X), y)
 
     assert_array_equal(clf.support_vectors_, sp_clf.support_vectors_.todense())
     assert_array_equal(clf.dual_coef_, sp_clf.dual_coef_.todense())
 
 
-def test_sparse_scale_C():
-    """Check that sparse LibSVM/LibLinear works ok with scaling of C"""
+def test_sparse_svc_clone_with_callable_kernel():
+    a = svm.SVC(C=1, kernel=lambda x, y: x * y.T, probability=True)
+    b = base.clone(a)
 
-    params = dict(kernel='linear', C=0.1)
-    klasses = [(svm.SVC, svm.sparse.SVC, params),
-               (svm.SVR, svm.sparse.SVR, params),
-               (svm.NuSVR, svm.sparse.NuSVR, params),
-               (svm.LinearSVC, svm.sparse.LinearSVC, {}),
-               (linear_model.LogisticRegression,
-                    linear_model.sparse.LogisticRegression, {})
-              ]
-
-    for klass, sparse_klass, this_params in klasses:
-        clf = klass(scale_C=True, **this_params).fit(X, Y)
-        clf_no_scale = klass(scale_C=False, **this_params).fit(X, Y)
-        sp_clf = sparse_klass(scale_C=True, **this_params).fit(X, Y)
-
-        sp_clf_coef_ = sp_clf.coef_
-        if sparse.issparse(sp_clf_coef_):
-            sp_clf_coef_ = sp_clf_coef_.todense()
-        assert_array_almost_equal(clf.coef_, sp_clf_coef_, 5)
-
-        error_with_scale = linalg.norm(clf_no_scale.coef_
-                           - sp_clf_coef_) / linalg.norm(clf_no_scale.coef_)
-        assert_true(error_with_scale > 1e-3)
+    b.fit(X_sp, Y)
+    b.predict(X_sp)
+    b.predict_proba(X_sp)
+    # b.decision_function(X_sp)  # XXX : should be supported
 
 
 if __name__ == '__main__':
