@@ -1,6 +1,3 @@
-#emacs: -*- mode: python-mode; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
-#ex: set sts=4 ts=4 sw=4 noet:
-
 # Basic unittests to test functioning of module's top-level
 
 __author__ = 'Yaroslav Halchenko'
@@ -12,9 +9,10 @@ from nose.tools import assert_equal
 try:
     from sklearn import *
     _top_import_error = None
-except Exception, e:
+except Exception as e:
     _top_import_error = e
 
+
 def test_import_skl():
     """Test either above import has failed for some reason
 
