@@ -7,16 +7,21 @@ from nose.tools import assert_almost_equal
 from nose.tools import assert_true
 from nose.tools import assert_raises
 
+from sklearn.utils.testing import assert_greater
 from sklearn.multiclass import OneVsRestClassifier
 from sklearn.multiclass import OneVsOneClassifier
 from sklearn.multiclass import OutputCodeClassifier
 from sklearn.svm import LinearSVC
 from sklearn.naive_bayes import MultinomialNB
+from sklearn.linear_model import LinearRegression, Lasso, ElasticNet, Ridge
+from sklearn.tree import DecisionTreeClassifier
 from sklearn.grid_search import GridSearchCV
+from sklearn import svm
 from sklearn import datasets
 
 iris = datasets.load_iris()
-perm = np.random.permutation(iris.target.size)
+rng = np.random.RandomState(0)
+perm = rng.permutation(iris.target.size)
 iris.data = iris.data[perm]
 iris.target = iris.target[perm]
 n_classes = 3
@@ -76,7 +81,9 @@ def test_ovr_multilabel():
                   [1, 0, 1],
                   [1, 0, 0]])
 
-    for base_clf in (MultinomialNB(), LinearSVC()):
+    for base_clf in (MultinomialNB(), LinearSVC(),
+                     LinearRegression(), Ridge(),
+                     ElasticNet(), Lasso(alpha=0.5)):
         # test input as lists of tuples
         clf = OneVsRestClassifier(base_clf).fit(X, y)
         y_pred = clf.predict([[0, 4, 4]])[0]
@@ -90,6 +97,13 @@ def test_ovr_multilabel():
         assert_true(clf.multilabel_)
 
 
+def test_ovr_fit_predict_svc():
+    ovr = OneVsRestClassifier(svm.SVC())
+    ovr.fit(iris.data, iris.target)
+    assert_equal(len(ovr.estimators_), 3)
+    assert_greater(ovr.score(iris.data, iris.target), .9)
+
+
 def test_ovr_multilabel_dataset():
     base_clf = MultinomialNB(alpha=1)
     for au, prec, recall in zip((True, False), (0.65, 0.74), (0.72, 0.84)):
@@ -120,6 +134,26 @@ def test_ovr_gridsearch():
     assert_true(best_C in Cs)
 
 
+def test_ovr_coef_():
+    ovr = OneVsRestClassifier(LinearSVC())
+    ovr.fit(iris.data, iris.target)
+    shape = ovr.coef_.shape
+    assert_equal(shape[0], n_classes)
+    assert_equal(shape[1], iris.data.shape[1])
+
+
+def test_ovr_coef_exceptions():
+    # Not fitted exception!
+    ovr = OneVsRestClassifier(LinearSVC())
+    # lambda is needed because we don't want coef_ to be evaluated right away
+    assert_raises(ValueError, lambda x: ovr.coef_, None)
+
+    # Doesn't have coef_ exception!
+    ovr = OneVsRestClassifier(DecisionTreeClassifier())
+    ovr.fit(iris.data, iris.target)
+    assert_raises(AttributeError, lambda x: ovr.coef_, None)
+
+
 def test_ovo_exceptions():
     ovo = OneVsOneClassifier(LinearSVC())
     assert_raises(ValueError, ovo.predict, [])
