@@ -9,28 +9,23 @@ Y = d.target
 
 
 def test_pls():
-    n_components = 2
     # 1) Canonical (symetric) PLS (PLS 2 blocks canonical mode A)
     # ===========================================================
     # Compare 2 algo.: nipals vs. svd
     # ------------------------------
-    pls_bynipals = pls.PLSCanonical(n_components=n_components)
+    pls_bynipals = pls.PLSCanonical(n_components=X.shape[1])
     pls_bynipals.fit(X, Y)
-    pls_bysvd = pls.PLSCanonical(algorithm="svd", n_components=n_components)
+    pls_bysvd = pls.PLSCanonical(algorithm="svd", n_components=X.shape[1])
     pls_bysvd.fit(X, Y)
-    # check that the loading vectors are highly correlated
+    # check equalities of loading (up to the sign of the second column)
     assert_array_almost_equal(
-        [np.abs(np.corrcoef(pls_bynipals.x_loadings_[:, k],
-                               pls_bysvd.x_loadings_[:, k])[1, 0])
-                           for k in xrange(n_components)],
-        np.ones(n_components),
+        pls_bynipals.x_loadings_,
+        np.multiply(pls_bysvd.x_loadings_, np.array([1, -1, 1])), decimal=5,
         err_msg="nipals and svd implementation lead to different x loadings")
 
     assert_array_almost_equal(
-        [np.abs(np.corrcoef(pls_bynipals.y_loadings_[:, k],
-                               pls_bysvd.y_loadings_[:, k])[1, 0])
-                           for k in xrange(n_components)],
-        np.ones(n_components),
+        pls_bynipals.y_loadings_,
+        np.multiply(pls_bysvd.y_loadings_, np.array([1, -1, 1])), decimal=5,
         err_msg="nipals and svd implementation lead to different y loadings")
 
     # Check PLS properties (with n_components=X.shape[1])
@@ -81,65 +76,58 @@ def test_pls():
 
     # "Non regression test" on canonical PLS
     # --------------------------------------
-    pls_bynipals = pls.PLSCanonical(n_components=n_components)
-    pls_bynipals.fit(X, Y)
-
-    pls_ca = pls_bynipals
-    x_loadings = np.array(
-       [[-0.61470416,  0.37877695],
-        [-0.65625755,  0.01196893],
-        [-0.51733059, -0.93984954]])
-    assert_array_almost_equal(pls_ca.x_loadings_, x_loadings)
-
-    y_loadings = np.array(
-        [[0.66591533,  0.77358148],
-         [0.67602364, -0.62871191],
-         [-0.35892128, -0.11981924]])
-    assert_array_almost_equal(pls_ca.y_loadings_, y_loadings)
+    pls_ca = pls.PLSCanonical(n_components=X.shape[1])
+    pls_ca.fit(X, Y)
 
     x_weights = np.array(
-        [[-0.61330704,  0.25616119],
-         [-0.74697144,  0.11930791],
-         [-0.25668686, -0.95924297]])
+        [[-0.61330704,  0.25616119, -0.74715187],
+         [-0.74697144,  0.11930791,  0.65406368],
+         [-0.25668686, -0.95924297, -0.11817271]])
     assert_array_almost_equal(pls_ca.x_weights_, x_weights)
 
+    x_rotations = np.array(
+        [[-0.61330704,  0.41591889, -0.62297525],
+         [-0.74697144,  0.31388326,  0.77368233],
+         [-0.25668686, -0.89237972, -0.24121788]])
+    assert_array_almost_equal(pls_ca.x_rotations_, x_rotations)
+
     y_weights = np.array(
-        [[0.58989127,  0.7890047],
-         [0.77134053, -0.61351791],
-         [-0.2388767, -0.03267062]])
+        [[+0.58989127,  0.7890047,   0.1717553],
+         [+0.77134053, -0.61351791,  0.16920272],
+         [-0.23887670, -0.03267062,  0.97050016]])
     assert_array_almost_equal(pls_ca.y_weights_, y_weights)
 
+    y_rotations = np.array(
+        [[+0.58989127,  0.7168115,  0.30665872],
+         [+0.77134053, -0.70791757,  0.19786539],
+         [-0.23887670, -0.00343595,  0.94162826]])
+    assert_array_almost_equal(pls_ca.y_rotations_, y_rotations)
+
     # 2) Regression PLS (PLS2): "Non regression test"
     # ===============================================
-    pls2 = pls.PLSRegression(n_components=n_components)
-    pls2.fit(X, Y)
-
-    x_loadings = np.array(
-        [[-0.61470416, -0.24574278],
-         [-0.65625755, -0.14396183],
-         [-0.51733059,  1.00609417]])
-    assert_array_almost_equal(pls2.x_loadings_, x_loadings)
-
-    y_loadings = np.array(
-        [[0.32456184,  0.29892183],
-         [0.42439636,  0.61970543],
-         [-0.13143144, -0.26348971]])
-    assert_array_almost_equal(pls2.y_loadings_, y_loadings)
+    pls_2 = pls.PLSRegression(n_components=X.shape[1])
+    pls_2.fit(X, Y)
 
     x_weights = np.array(
-        [[-0.61330704, -0.00443647],
-         [-0.74697144, -0.32172099],
-         [-0.25668686,  0.94682413]])
-    assert_array_almost_equal(pls2.x_weights_, x_weights)
+        [[-0.61330704, -0.00443647,  0.78983213],
+         [-0.74697144, -0.32172099, -0.58183269],
+         [-0.25668686,  0.94682413, -0.19399983]])
+    assert_array_almost_equal(pls_2.x_weights_, x_weights)
+
+    x_loadings = np.array(
+        [[-0.61470416, -0.24574278,  0.78983213],
+         [-0.65625755, -0.14396183, -0.58183269],
+         [-0.51733059,  1.00609417, -0.19399983]])
+    assert_array_almost_equal(pls_2.x_loadings_, x_loadings)
 
     y_weights = np.array(
-        [[0.58989127,  0.40572461],
-         [0.77134053,  0.84112205],
-         [-0.2388767, -0.35763282]])
-    assert_array_almost_equal(pls2.y_weights_, y_weights)
+        [[+0.32456184,  0.29892183,  0.20316322],
+         [+0.42439636,  0.61970543,  0.19320542],
+         [-0.13143144, -0.26348971, -0.17092916]])
+    assert_array_almost_equal(pls_2.y_weights_, y_weights)
 
-    ypred_2 = np.array(
-        [[180.33278555,   35.57034871,   56.06817703],
-         [192.06235219,   37.95306771,   54.12925192]])
-
-    assert_array_almost_equal(pls2.predict(X[:2]), ypred_2)
+    y_loadings = np.array(
+        [[+0.32456184,  0.29892183,  0.20316322],
+         [+0.42439636,  0.61970543,  0.19320542],
+         [-0.13143144, -0.26348971, -0.17092916]])
+    assert_array_almost_equal(pls_2.y_loadings_, y_loadings)
