@@ -1,12 +1,18 @@
+import os
+
 import numpy
 from numpy.distutils.misc_util import Configuration
 
 
 def configuration(parent_package="", top_path=None):
     config = Configuration("tree", parent_package, top_path)
+    libraries = []
+    if os.name == 'posix':
+        libraries.append('m')
     config.add_extension("_tree",
                          sources=["_tree.c"],
-                         include_dirs=[numpy.get_include()])
+                         include_dirs=[numpy.get_include()],
+                         libraries=libraries)
 
     config.add_subpackage("tests")
 
