@@ -1,4 +1,6 @@
+import os
 from os.path import join
+
 from numpy.distutils.system_info import get_info
 
 
@@ -19,11 +21,17 @@ def configuration(parent_package='', top_path=None):
     else:
         cblas_libs = blas_info.pop('libraries', [])
 
+    libraries = []
+    if os.name == 'posix':
+        libraries.append('m')
+        cblas_libs.append('m')
+
     config.add_extension('arraybuilder',
          sources=['arraybuilder.c'])
 
     config.add_extension('sparsefuncs',
-         sources=['sparsefuncs.c'])
+         sources=['sparsefuncs.c'],
+         libraries=libraries)
 
     config.add_extension('arrayfuncs',
          sources=['arrayfuncs.c'],
@@ -36,8 +44,27 @@ def configuration(parent_package='', top_path=None):
          **blas_info
          )
 
+    config.add_extension(
+        'murmurhash',
+        sources=['murmurhash.c', join('src', 'MurmurHash3.cpp')],
+        include_dirs=['src'])
+
     config.add_extension('graph_shortest_path',
          sources=['graph_shortest_path.c'],
          include_dirs=[numpy.get_include()])
 
+    config.add_extension('seq_dataset',
+         sources=['seq_dataset.c'],
+         include_dirs=[numpy.get_include()])
+
+    config.add_extension('weight_vector',
+         sources=['weight_vector.c'],
+         include_dirs=[numpy.get_include()],
+         libraries=libraries)
+
     return config
+
+
+if __name__ == '__main__':
+    from numpy.distutils.core import setup
+    setup(**configuration(top_path='').todict())
